@@ -28,7 +28,7 @@
 #include "libbalsa-marshal.h"
 
 #include "libbalsa-conf.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 
 static GObjectClass *parent_class = NULL;
 
