@@ -29,6 +29,7 @@
 #include <gmime/gmime.h>
 #include <gdk/gdk.h>
 
+#include "libbalsa-vfs.h"
 #include "config.h"
 
 #ifdef HAVE_GPGME
@@ -57,10 +58,14 @@ enum _LibBalsaAttachMode {
     LIBBALSA_ATTACH_AS_EXTBODY
 };
 
+#define LIBBALSA_MESSAGE_BODY_SAFE (S_IRUSR | S_IWUSR)
+#define LIBBALSA_MESSAGE_BODY_UNSAFE \
+    (S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH)
 struct _LibBalsaMessageBody {
     LibBalsaMessage *message;	/* The message of which this is a part */
     /* FIXME: remove buffer and buf_len to decrease memory usage. */
     gchar *buffer;		/* holds raw data of the MIME part, or NULL */
+    gchar *html_buffer;         /* holds the html representation of the part or NULL */
     ssize_t buflen;             /* size of the block */
     LibBalsaMessageHeaders *embhdrs;  /* headers of a message/rfc822 part */
     LibBalsaMessageBodyType body_type;
@@ -70,6 +75,7 @@ struct _LibBalsaMessageBody {
     const gchar *content_dsp;	/* content-disposition */ 
     const gchar *content_id;    /* content-id */
     gchar *filename;		/* holds filename for attachments and such (used mostly for sending) */
+    LibbalsaVfs * file_uri;     /* file uri for attachments (used for sending) */
     LibBalsaAttachMode attach_mode; /* attachment mode for sending */
     gchar *temp_filename;	/* Holds the filename of a the temporary file where this part is saved */
     gchar *charset;		/* the charset, used for sending, replying. */
@@ -93,18 +99,27 @@ LibBalsaMessageBodyType libbalsa_message_body_type(LibBalsaMessageBody *
 void libbalsa_message_body_set_mime_body(LibBalsaMessageBody * body,
 					 GMimeObject * mime_part);
 
-GMimeStream *libbalsa_message_body_get_stream(LibBalsaMessageBody * body);
+GMimeStream *libbalsa_message_body_get_stream(LibBalsaMessageBody * body,
+                                              GError **err);
 gssize libbalsa_message_body_get_content(LibBalsaMessageBody * body,
-                                         gchar ** buf);
+                                         gchar ** buf, GError **err);
 GdkPixbuf *libbalsa_message_body_get_pixbuf(LibBalsaMessageBody * body,
                                             GError ** err);
 
-gboolean libbalsa_message_body_save_fd(LibBalsaMessageBody * body, int fd,
-                                       gboolean filter_crlf);
+gboolean libbalsa_message_body_save_stream(LibBalsaMessageBody * body,
+                                           GMimeStream * dest,
+                                           gboolean filter_crlf,
+                                           GError **err);
 gboolean libbalsa_message_body_save(LibBalsaMessageBody * body,
-                                    const gchar * filename,
-                                    gboolean filter_crlf);
-gboolean libbalsa_message_body_save_temporary(LibBalsaMessageBody * body);
+                                    const gchar * filename, mode_t mode,
+                                    gboolean filter_crlf, GError **err);
+gboolean libbalsa_message_body_save_vfs(LibBalsaMessageBody * body,
+                                        LibbalsaVfs * dest,
+                                        mode_t mode,
+                                        gboolean filter_crlf,
+                                        GError **err);
+gboolean libbalsa_message_body_save_temporary(LibBalsaMessageBody * body,
+                                              GError **err);
 
 gchar *libbalsa_message_body_get_parameter(LibBalsaMessageBody * body,
 					   const gchar * param);
@@ -119,5 +134,9 @@ LibBalsaMessageBody *libbalsa_message_body_get_by_id(LibBalsaMessageBody *
                                                      body,
                                                      const gchar * id);
 
+#ifdef HAVE_GPGME
+LibBalsaMsgProtectState libbalsa_message_body_protect_state(LibBalsaMessageBody *body);
+#endif
+
 
 #endif				/* __LIBBALSA_BODY_H__ */
