@@ -385,7 +385,11 @@ libbalsa_condition_regex_free(LibBalsaConditionRegex* reg, gpointer throwaway)
 
     g_free(reg->string);
     if (reg->compiled) 
+#if USE_GREGEX
+        g_regex_unref(reg->compiled);
+#else                           /* USE_GREGEX */
 	regfree(reg->compiled);
+#endif                          /* USE_GREGEX */
 }				/* end condition_regex_free() */
 
 void 
@@ -477,12 +481,20 @@ compare_regexs(GSList * c1,GSList * c2)
 /* Helper to compare conditions, a bit obscure at first glance
    but we have to compare complex structure, so we must check
    all fields.
+   NULL conditions are OK, and compare equal only if both are NULL.
 */
 
 gboolean
 libbalsa_condition_compare(LibBalsaCondition *c1,LibBalsaCondition *c2)
 {
     gboolean res = FALSE;
+
+    if (c1 == c2) 
+        return TRUE;
+
+    if (c1 == NULL || c2 == NULL)
+        return FALSE;
+
     switch (c1->type) {
     case CONDITION_STRING:
         res = (c2->type == CONDITION_STRING ||
