@@ -28,12 +28,14 @@
 #ifndef __FILTER_PRIVATE_H__
 #define __FILTER_PRIVATE_H__
 
-#ifdef HAVE_PCRE
-#  include <pcreposix.h>
-#else
-#  include <sys/types.h>
-#  include <regex.h>
-#endif
+#if !USE_GREGEX
+#  ifdef HAVE_PCRE
+#    include <pcreposix.h>
+#  else
+#    include <sys/types.h>
+#    include <regex.h>
+#  endif
+#endif                          /* USE_GREGEX */
 
 
 /* regex options */
@@ -43,7 +45,11 @@
 /* regex struct */
 struct _LibBalsaConditionRegex {
     gchar *string;
+#if USE_GREGEX
+    GRegex *compiled;
+#else                           /* USE_GREGEX */
     regex_t *compiled;
+#endif                          /* USE_GREGEX */
 };
 
 #endif				/* __FILTER_PRIVATE_H__ */
