@@ -28,6 +28,7 @@
 #ifdef HAVE_GPGME
 #  include "rfc3156.h"
 #endif
+#include "libbalsa.h"
 #include "identity.h"
 #include "information.h"
 #include "libbalsa-conf.h"
@@ -101,6 +102,7 @@ libbalsa_identity_init(LibBalsaIdentity* ident)
     ident->bcc = NULL;
     ident->reply_string = g_strdup(_("Re:"));
     ident->forward_string = g_strdup(_("Fwd:"));
+    ident->send_mp_alternative = FALSE;
     ident->signature_path = NULL;
     ident->sig_executable = FALSE;
     ident->sig_sending = TRUE;
@@ -112,6 +114,7 @@ libbalsa_identity_init(LibBalsaIdentity* ident)
     ident->gpg_sign = FALSE;
     ident->gpg_encrypt = FALSE;
     ident->always_trust = FALSE;
+    ident->warn_send_plain = TRUE;
     ident->crypt_protocol = LIBBALSA_PROTECT_OPENPGP;
 #endif
     ident->request_mdn = FALSE;
@@ -249,6 +252,14 @@ libbalsa_identity_set_forward_string(LibBalsaIdentity* ident, const gchar* forwa
 }
 
 
+void
+libbalsa_identity_set_send_mp_alternative(LibBalsaIdentity* ident, gboolean send_mp_alternative)
+{
+    g_return_if_fail(ident != NULL);
+    ident->send_mp_alternative = send_mp_alternative;
+}
+
+
 void
 libbalsa_identity_set_signature_path(LibBalsaIdentity* ident, const gchar* path)
 {
@@ -627,6 +638,10 @@ static void ident_dialog_add_gpg_menu(GtkWidget * table, gint row,
                                       GtkDialog * dialog,
                                       const gchar * label_name,
                                       const gchar * menu_key);
+static void add_show_menu(const char *label, gpointer data,
+                          GtkWidget * menu);
+static void ident_dialog_free_values(GPtrArray * values);
+
 #ifdef HAVE_GPGME
 static void display_frame_set_gpg_mode(GObject * dialog,
                                        const gchar * key, gint * value);
@@ -644,11 +659,8 @@ static void display_frame_set_server(GObject * dialog,
 #endif /* ENABLE_ESMTP */
 
 #if defined(HAVE_GPGME) || ENABLE_ESMTP
-static void add_show_menu(const char *label, gpointer data,
-                          GtkWidget * menu);
 static gpointer ident_dialog_get_value(GObject * dialog,
                                        const gchar * key);
-static void ident_dialog_free_values(GPtrArray * values);
 #endif                          /* defined(HAVE_GPGME) || ENABLE_ESMTP */
 
 /* Callback for the "toggled" signal of the "Default" column. */
@@ -979,7 +991,7 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
                            "identity-domain");
 
     /* create the "Messages" tab */
-    table = append_ident_notebook_page(notebook, 8, _("Messages"), NULL);
+    table = append_ident_notebook_page(notebook, 9, _("Messages"), NULL);
     row = 0;
     ident_dialog_add_entry(table, row++, dialog, _("_Bcc:"), 
                            "identity-bcc");
@@ -987,6 +999,9 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
                            "identity-replystring");
     ident_dialog_add_entry(table, row++, dialog, _("F_orward String:"), 
                            "identity-forwardstring");
+    ident_dialog_add_checkbutton(table, row++, dialog, 
+                                 _("send messages in both plain text and _HTML format"),
+                                 "identity-sendmpalternative", TRUE);
     ident_dialog_add_checkbutton(table, row++, dialog, 
                                  _("request _Message Disposition Notification by default"),
                                  "identity-requestmdn", TRUE);
@@ -1035,7 +1050,7 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
 #endif
     /* create the "Security" tab */
     table =
-        append_ident_notebook_page(notebook, 4, _("Security"), footnote);
+        append_ident_notebook_page(notebook, 5, _("Security"), footnote);
     row = 0;
     ident_dialog_add_checkbutton(table, row++, dialog, 
                                  _("sign messages by default"),
@@ -1049,6 +1064,9 @@ setup_ident_frame(GtkDialog * dialog, gboolean createp, gpointer tree)
     ident_dialog_add_checkbutton(table, row++, dialog,
                                  _("always trust GnuPG keys when encrypting"),
                                  "identity-trust-always", TRUE);
+    ident_dialog_add_checkbutton(table, row++, dialog,
+                                 _("remind me if messages can be encrypted"),
+                                 "identity-warn-send-plain", TRUE);
 #ifndef HAVE_GPGME
     gtk_widget_set_sensitive(table, FALSE);
 #endif
@@ -1463,6 +1481,7 @@ ident_dialog_update(GObject * dlg)
     id->reply_string    = ident_dialog_get_text(dlg, "identity-replystring");
     g_free(id->forward_string);
     id->forward_string  = ident_dialog_get_text(dlg, "identity-forwardstring");
+    id->send_mp_alternative = ident_dialog_get_bool(dlg, "identity-sendmpalternative");
 #if ENABLE_ESMTP
     if(id->smtp_server) g_object_unref(id->smtp_server);
     id->smtp_server = ident_dialog_get_value(dlg, "identity-smtp-server");
@@ -1489,6 +1508,7 @@ ident_dialog_update(GObject * dlg)
     id->gpg_sign        = ident_dialog_get_bool(dlg, "identity-gpgsign");
     id->gpg_encrypt     = ident_dialog_get_bool(dlg, "identity-gpgencrypt");
     id->always_trust    = ident_dialog_get_bool(dlg, "identity-trust-always");
+    id->warn_send_plain = ident_dialog_get_bool(dlg, "identity-warn-send-plain");
     id->crypt_protocol  = GPOINTER_TO_INT(ident_dialog_get_value
                                           (dlg, "identity-crypt-protocol"));
 #endif
@@ -1829,6 +1849,8 @@ display_frame_update(GObject * dialog, LibBalsaIdentity* ident)
                             ident->reply_string);
     display_frame_set_field(dialog, "identity-forwardstring", 
                             ident->forward_string);
+    display_frame_set_boolean(dialog, "identity-sendmpalternative",
+                              ident->send_mp_alternative);
 #if ENABLE_ESMTP
     display_frame_set_server(dialog, "identity-smtp-server",
                              ident->smtp_server);
@@ -1869,6 +1891,8 @@ display_frame_update(GObject * dialog, LibBalsaIdentity* ident)
                               ident->gpg_encrypt);    
     display_frame_set_boolean(dialog, "identity-trust-always", 
                               ident->always_trust);    
+    display_frame_set_boolean(dialog, "identity-warn-send-plain", 
+                              ident->warn_send_plain);    
     display_frame_set_gpg_mode(dialog, "identity-crypt-protocol",
 			   &ident->crypt_protocol);
 #endif
@@ -1953,6 +1977,8 @@ libbalsa_identity_new_config(const gchar* name)
         g_free(ident->forward_string);
         ident->forward_string = tmpstr;
     }
+    ident->send_mp_alternative =
+        libbalsa_conf_get_bool("SendMultipartAlternative");
     
     ident->signature_path = libbalsa_conf_get_string("SignaturePath");
     ident->sig_executable = libbalsa_conf_get_bool("SigExecutable");
@@ -1969,6 +1995,7 @@ libbalsa_identity_new_config(const gchar* name)
     ident->gpg_sign = libbalsa_conf_get_bool("GpgSign");
     ident->gpg_encrypt = libbalsa_conf_get_bool("GpgEncrypt");
     ident->always_trust = libbalsa_conf_get_bool("GpgTrustAlways");
+    ident->warn_send_plain = libbalsa_conf_get_bool("GpgWarnSendPlain=true");
     ident->crypt_protocol = libbalsa_conf_get_int("CryptProtocol=16");
 #endif
 
@@ -1991,6 +2018,7 @@ libbalsa_identity_save(LibBalsaIdentity* ident, const gchar* group)
     libbalsa_conf_set_string("Bcc", ident->bcc);
     libbalsa_conf_set_string("ReplyString", ident->reply_string);
     libbalsa_conf_set_string("ForwardString", ident->forward_string);
+    libbalsa_conf_set_bool("SendMultipartAlternative", ident->send_mp_alternative);
 #if ENABLE_ESMTP
     libbalsa_conf_set_string("SmtpServer",
                              libbalsa_smtp_server_get_name(ident->
@@ -2014,6 +2042,7 @@ libbalsa_identity_save(LibBalsaIdentity* ident, const gchar* group)
     libbalsa_conf_set_bool("GpgSign", ident->gpg_sign);
     libbalsa_conf_set_bool("GpgEncrypt", ident->gpg_encrypt);
     libbalsa_conf_set_bool("GpgTrustAlways", ident->always_trust);
+    libbalsa_conf_set_bool("GpgWarnSendPlain", ident->warn_send_plain);
     libbalsa_conf_set_int("CryptProtocol", ident->crypt_protocol);
 #endif
 
@@ -2112,6 +2141,24 @@ ident_dialog_add_gpg_menu(GtkWidget * table, gint row, GtkDialog * dialog,
 #endif
 }
 
+/* add_show_menu: helper function */
+static void
+add_show_menu(const char *label, gpointer data, GtkWidget * menu)
+{
+    GPtrArray *values =
+        g_object_get_data(G_OBJECT(menu), "identity-value");
+
+    gtk_combo_box_append_text(GTK_COMBO_BOX(menu), label);
+    g_ptr_array_add(values, data);
+}
+
+/* ident_dialog_free_values: helper function */
+static void
+ident_dialog_free_values(GPtrArray * values)
+{
+    g_ptr_array_free(values, TRUE);
+}
+
 #if ENABLE_ESMTP
 static void
 ident_dialog_add_smtp_menu(GtkWidget * table, gint row, GtkDialog * dialog,
@@ -2162,17 +2209,6 @@ display_frame_set_server(GObject * dialog, const gchar * key,
 #endif                          /* ENABLE_ESMTP */
 
 #if defined(HAVE_GPGME) || ENABLE_ESMTP
-/* add_show_menu: helper function */
-static void
-add_show_menu(const char *label, gpointer data, GtkWidget * menu)
-{
-    GPtrArray *values =
-        g_object_get_data(G_OBJECT(menu), "identity-value");
-
-    gtk_combo_box_append_text(GTK_COMBO_BOX(menu), label);
-    g_ptr_array_add(values, data);
-}
-
 /*
  * Get the value of the active option menu item
  */
@@ -2189,11 +2225,5 @@ ident_dialog_get_value(GObject * dialog, const gchar * key)
     
     return g_ptr_array_index(values, value);
 }
-
-/* ident_dialog_free_values: helper function */
-static void
-ident_dialog_free_values(GPtrArray * values)
-{
-    g_ptr_array_free(values, TRUE);
-}
 #endif /* defined(HAVE_GPGME) || ENABLE_ESMTP */
+
