@@ -106,7 +106,7 @@ imap_auth_gssapi(ImapMboxHandle* handle)
                         "GASSAPI", &user, &ok);
     if(!ok || user == NULL) {
         imap_mbox_handle_set_msg(handle, "Authentication cancelled");
-        return IMAP_AUTH_FAILURE;
+        return IMAP_AUTH_CANCELLED;
     }
     
     /* start the negotiation */
@@ -258,7 +258,7 @@ ag_negotiate_parameters(ImapMboxHandle *handle, const char * user,
     gss_buffer_desc request_buf, send_token;
     char buf[LONG_STRING];
     gss_qop_t quality;
-    unsigned cflags;
+    int cflags;
     char server_conf_flags;
     unsigned char *t;
     unsigned long buf_size;
