@@ -30,59 +30,6 @@
 
 #define ELEMENTS(x) (sizeof (x) / sizeof(x[0]))
 
-typedef unsigned (*CoalesceFunc)(int, void*);
-static gchar*
-coalesce_seq_range(int lo, int hi, CoalesceFunc incl, void *data)
-{
-  GString * res = g_string_sized_new(16);
-  enum { BEGIN, LASTOUT, LASTIN, RANGE } mode = BEGIN;
-  int seq;
-  unsigned prev =0, num = 0;
-
-  for(seq=lo; seq<=hi+1; seq++) {
-    if(seq<=hi && (num=incl(seq, data)) != 0) {
-      switch(mode) {
-      case BEGIN: 
-        g_string_append_printf(res, "%u", num);
-        mode = LASTIN; break;
-      case RANGE:
-        if(num!=prev+1) {
-          g_string_append_printf(res, ":%u,%u", prev, num);
-          mode = LASTIN;
-        }
-        break;
-      case LASTIN: 
-        if(num==prev+1) {
-          mode = RANGE;
-          break;
-        } /* else fall through */
-      case LASTOUT: 
-        g_string_append_printf(res, ",%u", num);
-        mode = LASTIN; break;
-      }
-    } else {
-      switch(mode) {
-      case BEGIN:
-      case LASTOUT: break;
-      case LASTIN: mode = LASTOUT; break;
-      case RANGE: 
-        g_string_append_printf(res, ":%u", prev);
-        mode = LASTOUT;
-        break;
-      }
-    }
-    prev = num;
-  }
-  return g_string_free(res, mode == BEGIN);
-}
-
-static unsigned
-simple_coealesce_func(int i, unsigned msgno[])
-{
-  return msgno[i];
-}
-
-
 struct fetch_data {
   ImapMboxHandle* h;
   ImapFetchType ift;
@@ -204,9 +151,13 @@ imap_mbox_handle_can_do(ImapMboxHandle* handle, ImapCapability cap)
 ImapResponse
 imap_mbox_handle_noop(ImapMboxHandle *handle)
 {
+  ImapResponse rc;
   IMAP_REQUIRED_STATE3(handle, IMHS_CONNECTED, IMHS_AUTHENTICATED,
                        IMHS_SELECTED, IMR_BAD);
-  return imap_cmd_exec(handle, "NOOP");
+  if(HANDLE_TRYLOCK(handle) != 0) return IMR_OK;
+  rc = imap_cmd_exec(handle, "NOOP");
+  HANDLE_UNLOCK(handle);
+  return rc;
 }
 
 /* 6.1.3 LOGOUT Command */
@@ -440,69 +391,229 @@ enum_flag_to_str(ImapMsgFlags flg)
   return g_string_free(flags_str, FALSE);
 }
 
+struct SingleAppendData {
+  ImapAppendFunc cb;
+  void *cb_data;
+  size_t size;
+  ImapMsgFlags flags;
+};
+
+static size_t
+single_append_cb(char *buf, size_t buf_sz, ImapAppendMultiStage stage,
+		 ImapMsgFlags *f, void *arg)
+{
+  struct SingleAppendData *sad = (struct SingleAppendData*)arg;
+  size_t sz;
+  switch(stage) {
+  case IMA_STAGE_NEW_MSG:
+    *f = sad->flags; 
+    sz = sad->size; sad->size = 0;
+    return sz;
+  case IMA_STAGE_PASS_DATA:
+    return sad->cb(buf, buf_sz, sad->cb_data);
+  }
+  g_assert_not_reached();
+  return 0;
+}
+ 
 ImapResponse
 imap_mbox_append(ImapMboxHandle *handle, const char *mbox,
                  ImapMsgFlags flags, size_t sz,
                  ImapAppendFunc dump_cb, void *arg)
 {
+  struct SingleAppendData sad = { dump_cb, arg, sz, flags };
   IMAP_REQUIRED_STATE2(handle,IMHS_AUTHENTICATED, IMHS_SELECTED, IMR_BAD);
-  {
-  int use_literal = imap_mbox_handle_can_do(handle, IMCAP_LITERAL);
-  unsigned cmdno;
+  return imap_mbox_append_multi(handle, mbox, single_append_cb, &sad, NULL);
+}
+
+static ImapResponse
+append_commit(ImapMboxHandle *handle, unsigned cmdno, ImapSequence *uid_seq)
+{
   ImapResponse rc;
-  gchar *mbx7 = imap_utf8_to_mailbox(mbox);
-  gchar *cmd;
-  char *litstr = use_literal ? "+" : "";
-  char buf[8192];
-  size_t s, delta;
-  int c;
 
-  imap_handle_idle_disable(handle);
-  if(flags) {
-    gchar *str = enum_flag_to_str(flags);
-    cmd = g_strdup_printf("APPEND \"%s\" (%s) {%u%s}", mbx7, str,
-                          (unsigned)sz, litstr);
-    g_free(str);
-  } else 
-    cmd = g_strdup_printf("APPEND \"%s\" {%u%s}", mbx7, (unsigned)sz, litstr);
+  if(handle->uidplus.dst)
+    g_warning("Leaking memory in imap_append");
 
-  c = imap_cmd_start(handle, cmd, &cmdno);
-  g_free(mbx7); g_free(cmd);
-  if (c<0) /* irrecoverable connection error. */
-    return IMR_SEVERED;
-  if(use_literal)
-    rc = IMR_RESPOND; /* we do it without flushing */
-  else {
-    sio_flush(handle->sio);
-    do {
-      rc = imap_cmd_step (handle, cmdno);
-    } while (rc == IMR_UNTAGGED);
-    if(rc != IMR_RESPOND) return rc;
-    while( (c=sio_getc(handle->sio)) != -1 && c != '\n');
-  } 
-
-  for(s=0; s<sz; s+= delta) {
-    delta = dump_cb(buf, sizeof(buf), arg);
-    if(s+delta>sz) delta = sz-s;
-    sio_write(handle->sio, buf, delta);
+  if(uid_seq) {
+    handle->uidplus.dst = uid_seq->ranges;
+    handle->uidplus.store_response = 1;
   }
-  
-  /* It has been though observed that "Cyrus IMAP4 v2.0.16-p1
-   * server" can hang if the flush isn't done under following conditions:
-   * a). TLS is enabled, b). message contains NUL characters.  NUL
-   * characters are forbidden (RFC3501, sect. 4.3.1) and we probably
-   * should make sure on a higher level that they are not sent.
-   */
-  /* sio_flush(handle->sio); */
   sio_write(handle->sio, "\r\n", 2);
   sio_flush(handle->sio);
   do {
     rc = imap_cmd_step (handle, cmdno);
   } while (rc == IMR_UNTAGGED);
 
-  imap_handle_idle_enable(handle, 30);
-  return rc;
+  if(uid_seq) {
+    if(uid_seq->uid_validity == 0) {
+      uid_seq->uid_validity = handle->uidplus.dst_uid_validity;
+      uid_seq->ranges = handle->uidplus.dst;
+    } else if(uid_seq->uid_validity != handle->uidplus.dst_uid_validity) {
+      printf("The IMAP server keeps changing UID validity, "
+	     "ignoring UIDPLUS response (%u -> %u)\n",
+	     uid_seq->uid_validity, handle->uidplus.dst_uid_validity);
+      uid_seq->uid_validity = handle->uidplus.dst_uid_validity;
+      g_list_free(uid_seq->ranges);
+      uid_seq->ranges = NULL;
+    } else {
+      uid_seq->ranges = handle->uidplus.dst;
+    }
   }
+  handle->uidplus.dst = NULL;
+  handle->uidplus.store_response = 0;
+
+  return rc;
+}
+
+static ImapResponse
+imap_mbox_append_multi_real(ImapMboxHandle *handle,
+			    const char *mbox,
+			    ImapAppendMultiFunc dump_cb,
+			    void* cb_arg,
+			    ImapSequence *uid_sequence)
+{
+  static const unsigned TRANSACTION_SIZE = 10*1024*1024;
+  int use_literal, use_multiappend, use_uidplus;
+  unsigned cmdno;
+  ImapResponse rc = IMR_OK;
+  char *litstr;
+  char buf[16384];
+  size_t s, msg_size, delta, current_transaction_size = 0;
+  int c, msg_cnt;
+  ImapMsgFlags flags;
+  gboolean new_append = TRUE;
+
+  IMAP_REQUIRED_STATE2(handle,IMHS_AUTHENTICATED, IMHS_SELECTED, IMR_BAD);
+
+  use_literal = imap_mbox_handle_can_do(handle, IMCAP_LITERAL);
+  use_multiappend = imap_mbox_handle_can_do(handle, IMCAP_MULTIAPPEND);
+  use_uidplus = imap_mbox_handle_can_do(handle, IMCAP_UIDPLUS);
+  litstr = use_literal ? "+" : "";
+
+  if(uid_sequence)
+    uid_sequence->ranges = NULL;
+
+  imap_handle_idle_disable(handle);
+  for(msg_cnt=0;
+      (msg_size = dump_cb(buf, sizeof(buf),
+			  IMA_STAGE_NEW_MSG, &flags, cb_arg)) >0;
+      msg_cnt++) {
+
+    if (handle->state == IMHS_DISCONNECTED)
+      return IMR_SEVERED;
+  
+    if(new_append || !use_multiappend) {
+      gchar *cmd;
+      gchar *mbx7 = imap_utf8_to_mailbox(mbox);
+      new_append = FALSE;
+      if(flags) {
+	gchar *str = enum_flag_to_str(flags);
+	cmd = g_strdup_printf("APPEND \"%s\" (%s) {%lu%s}",
+			      mbx7, str, (unsigned long)msg_size, litstr);
+	g_free(str);
+      } else 
+	cmd = g_strdup_printf("APPEND \"%s\" {%lu%s}",
+			      mbx7, (unsigned long)msg_size, litstr);
+      c = imap_cmd_start(handle, cmd, &cmdno);
+      g_free(mbx7); g_free(cmd);
+
+      if (c<0) /* irrecoverable connection error. */
+	return IMR_SEVERED;
+
+    } else {
+      /* MULTIAPPEND continuation */
+      if(flags) {
+	gchar *str = enum_flag_to_str(flags);
+	sio_printf(handle->sio, " (%s) {%lu%s}\r\n", str,
+		   (unsigned long)msg_size, litstr);
+	g_free(str);
+      } else 
+	sio_printf(handle->sio, " {%lu%s}\r\n",
+		   (unsigned long)msg_size, litstr);
+    }
+
+    if(use_literal)
+      rc = IMR_RESPOND; /* we do it without flushing */
+    else {
+      sio_flush(handle->sio);
+      do {
+	rc = imap_cmd_step (handle, cmdno);
+      } while (rc == IMR_UNTAGGED);
+      if(rc != IMR_RESPOND) return rc;
+      while( (c=sio_getc(handle->sio)) != -1 && c != '\n');
+      if(c == -1) {
+	imap_handle_disconnect(handle);
+	return IMR_SEVERED;
+      }
+    }
+
+    for(s=0; s<msg_size; s+= delta) {
+      delta = dump_cb(buf, sizeof(buf), IMA_STAGE_PASS_DATA, NULL, cb_arg);
+      if(s+delta>msg_size) delta = msg_size-s;
+      sio_write(handle->sio, buf, delta);
+    }
+  
+    current_transaction_size += msg_size;
+    if(current_transaction_size > TRANSACTION_SIZE) {
+      current_transaction_size = 0;
+      new_append = TRUE;
+    }
+    if(new_append || !use_multiappend) { /* Grab the response. */
+      /* Data written, tie up the message.  It has been though
+       * observed that "Cyrus IMAP4 v2.0.16-p1 server" can hang if the
+       * flush isn't done under following conditions: a). TLS is
+       * enabled, b). message contains NUL characters.  NUL characters
+       * are forbidden (RFC3501, sect. 4.3.1) and we probably should
+       * make sure on a higher level that they are not sent.
+       */
+      /* sio_flush(handle->sio); */
+      if( (rc=append_commit(handle, cmdno,
+			   use_uidplus ? uid_sequence : NULL)) != IMR_OK)
+      return rc;
+    }
+    /* And move to the next message... */
+  }
+
+  if(!new_append && use_multiappend) { /* We get the server response here... */
+    rc = append_commit(handle, cmdno, use_uidplus ? uid_sequence : NULL);
+  }
+
+  imap_handle_idle_enable(handle, 30);
+
+  if(uid_sequence) {
+    uid_sequence->ranges = g_list_reverse(uid_sequence->ranges);
+  }
+  return rc;
+}
+
+
+/** Appends multiple messages at once, using MULTIAPPEND extension if
+    available.
+    @param handle the IMAP connection.
+
+    @param mbox the UTF-8 encoded mailbox name the messages are to be
+    appended to.
+
+    @param dump_cb function providing the data. It is called first
+    once for each message with IMA_STAGE_NEW_MSG stage parameter to
+    get the message size and message flags. Size zero indicates last
+    message. Next, it is called several times with IMA_STAGE_PASS_DATA
+    stage parameter to actually get the message data.
+
+    @param cb_arg the context passed to dump_cb.
+ */
+ImapResponse
+imap_mbox_append_multi(ImapMboxHandle *handle,
+		       const char *mbox,
+		       ImapAppendMultiFunc dump_cb,
+		       void* cb_arg,
+		       ImapSequence *uids)
+{
+  ImapResponse rc;
+  HANDLE_LOCK(handle);
+  rc = imap_mbox_append_multi_real(handle, mbox, dump_cb, cb_arg, uids);
+  HANDLE_UNLOCK(handle);
+  return rc;
 }
 
 #ifdef USE_IMAP_APPEND_STR	/* not used currently */
@@ -567,8 +678,13 @@ imap_mbox_close(ImapMboxHandle *h)
 ImapResponse
 imap_mbox_expunge(ImapMboxHandle *handle)
 {
+  ImapResponse rc;
+
   IMAP_REQUIRED_STATE2(handle,IMHS_AUTHENTICATED, IMHS_SELECTED, IMR_BAD);
-  return imap_cmd_exec(handle, "EXPUNGE");
+  HANDLE_LOCK(handle);
+  rc = imap_cmd_exec(handle, "EXPUNGE");
+  HANDLE_UNLOCK(handle);
+  return rc;
 }
 
 ImapResponse
@@ -727,8 +843,8 @@ imap_assure_needed_flags(ImapMboxHandle *h, ImapMsgFlag needed_flags)
           if( (f->known_flags & fnd.flag) ==0)
             f->flag_values |= fnd.flag;
         }
-      seqno = coalesce_seq_range(1, h->exists,
-                                 (CoalesceFunc)flag_unknown, &fnd);
+      seqno = imap_coalesce_seq_range(1, h->exists,
+				      (ImapCoalesceFunc)flag_unknown, &fnd);
       if(!seqno) /* oops, why were we called in the first place!? */
         continue;
       switch(fnd.flag) {
@@ -860,8 +976,9 @@ imap_mbox_handle_fetch_range(ImapMboxHandle* handle,
   gchar * seq;
   ImapResponse rc;
   unsigned exists = imap_mbox_handle_get_exists(handle);
-  CoalesceFunc cf = (CoalesceFunc)(mbox_view_is_active(&handle->mbox_view)
-                                   ? need_fetch_view : need_fetch);
+  ImapCoalesceFunc cf = 
+    (ImapCoalesceFunc)(mbox_view_is_active(&handle->mbox_view)
+		       ? need_fetch_view : need_fetch);
   struct fetch_data fd;
 
   IMAP_REQUIRED_STATE1(handle, IMHS_SELECTED, IMR_BAD);
@@ -870,7 +987,7 @@ imap_mbox_handle_fetch_range(ImapMboxHandle* handle,
   if(lo>hi) return IMR_OK;
   if(lo<1) lo = 1;
   if(hi>exists) hi = exists;
-  seq = coalesce_seq_range(lo, hi, cf, &fd);
+  seq = imap_coalesce_seq_range(lo, hi, cf, &fd);
   if(seq) {
     const char* hdr[13];
     ic_construct_header_list(hdr, fd.req_fetch_type);
@@ -889,7 +1006,7 @@ imap_mbox_handle_fetch_set(ImapMboxHandle* handle,
 {
   gchar * seq;
   ImapResponse rc;
-  CoalesceFunc cf;
+  ImapCoalesceFunc cf;
   struct fetch_data_set fd;
 
   IMAP_REQUIRED_STATE1(handle, IMHS_SELECTED, IMR_BAD);
@@ -897,9 +1014,9 @@ imap_mbox_handle_fetch_set(ImapMboxHandle* handle,
 
   fd.fd.h = handle; fd.fd.ift = fd.fd.req_fetch_type = ift;
   fd.set = set;
-  cf = (CoalesceFunc)(mbox_view_is_active(&handle->mbox_view)
-                      ? need_fetch_view_set : need_fetch_set);
-  seq = coalesce_seq_range(1, cnt, cf, &fd);
+  cf = (ImapCoalesceFunc)(mbox_view_is_active(&handle->mbox_view)
+			  ? need_fetch_view_set : need_fetch_set);
+  seq = imap_coalesce_seq_range(1, cnt, cf, &fd);
   if(seq) {
     const char* hdr[13];
     ic_construct_header_list(hdr, fd.fd.req_fetch_type);
@@ -913,16 +1030,34 @@ imap_mbox_handle_fetch_set(ImapMboxHandle* handle,
 }
 
 static void
-write_nstring(unsigned seqno, const char *str, int len, void *fl)
+write_nstring(unsigned seqno, ImapFetchBodyType body_type,
+              const char *str, size_t len, void *fl)
 {
-  if (fwrite(str, 1, len, (FILE*)fl) != (size_t) len)
+  if (fwrite(str, 1, len, (FILE*)fl) != len)
     perror("write_nstring");
 }
 
+struct FetchBodyPassthroughData {
+  ImapFetchBodyCb cb;
+  void *arg;
+};
+
+static void
+fetch_body_passthrough(unsigned seqno,
+		       ImapFetchBodyType body_type,
+		       const char *buf,
+		       size_t buflen, void* arg)
+{
+  struct FetchBodyPassthroughData* data = (struct FetchBodyPassthroughData*)arg;
+
+  data->cb(seqno, buf, buflen, data->arg);
+}
+
 ImapResponse
 imap_mbox_handle_fetch_rfc822(ImapMboxHandle* handle,
 			      unsigned cnt, unsigned *set,
-                              FILE *fl)
+			      ImapFetchBodyCb fetch_cb,
+			      void *fetch_cb_data)
 {
   ImapResponse rc = IMR_OK;
   gchar *seq;
@@ -930,14 +1065,17 @@ imap_mbox_handle_fetch_rfc822(ImapMboxHandle* handle,
   IMAP_REQUIRED_STATE1(handle, IMHS_SELECTED, IMR_BAD);
   HANDLE_LOCK(handle);
 
-  seq = coalesce_seq_range(0, cnt-1, (CoalesceFunc)simple_coealesce_func, set);
+  seq = imap_coalesce_set(cnt, set);
 
   if(seq) {
-    ImapFetchBodyCb cb = handle->body_cb;
-    void          *arg = handle->body_arg;
+    ImapFetchBodyInternalCb cb = handle->body_cb;
+    void                   *arg = handle->body_arg;
     gchar *cmd = g_strdup_printf("FETCH %s RFC822", seq);
-    handle->body_cb  = write_nstring;
-    handle->body_arg = fl;
+    struct FetchBodyPassthroughData passthrough_data;
+    passthrough_data.cb = fetch_cb;
+    passthrough_data.arg = fetch_cb_data;
+    handle->body_cb  = fetch_cb ? fetch_body_passthrough : NULL;
+    handle->body_arg = &passthrough_data;
     rc = imap_cmd_exec(handle, cmd);
     handle->body_cb  = cb;
     handle->body_arg = arg;
@@ -949,21 +1087,128 @@ imap_mbox_handle_fetch_rfc822(ImapMboxHandle* handle,
   return rc;
 }
 
+/* When peeking into message, we need to assure that we save header
+   first and body later: the order the fields are returned is in
+   principle undefined. */
+struct FetchBodyHeaderText {
+  FILE *out_file;
+  char *body;
+  size_t length;
+  gboolean wrote_header;
+};
+
+static void
+write_header_text_ordered(unsigned seqno, ImapFetchBodyType body_type,
+			  const char *str, size_t len, void *arg)
+{
+  struct FetchBodyHeaderText *fbht = (struct FetchBodyHeaderText*)arg;
+  switch(body_type) {
+  case IMAP_BODY_TYPE_RFC822:
+    g_warning("Server sends unrequested RFC822 response");
+    break; /* This is really unexpected response! */
+  case IMAP_BODY_TYPE_HEADER:
+    if (fwrite(str, 1, len, fbht->out_file) != len)
+      perror("write_nstring");
+    fbht->wrote_header = TRUE;
+    if(fbht->body) {
+      if (fwrite(fbht->body, 1, fbht->length, fbht->out_file) != fbht->length)
+	perror("write_nstring");
+      g_free(fbht->body); fbht->body = NULL;
+    }
+    break;
+  case IMAP_BODY_TYPE_TEXT:
+  case IMAP_BODY_TYPE_BODY:
+    if(fbht->wrote_header) {
+      if (fwrite(str, 1, len, fbht->out_file) != len)
+	perror("write_nstring");
+    } else {
+      fbht->body = g_malloc(len);
+      fbht->length = len;
+      memcpy(fbht->body, str, len);
+    }
+    break;
+  }
+}
+
+/* There is some point to code reuse here. */
+struct PassHeaderTextOrdered {
+  ImapFetchBodyCb cb;
+  void *arg;
+  char *body;
+  size_t length;
+  gboolean wrote_header;
+};
+
+static void
+pass_header_text_ordered(unsigned seqno, ImapFetchBodyType body_type,
+			  const char *str, size_t len, void *arg)
+{
+  struct PassHeaderTextOrdered *phto = (struct PassHeaderTextOrdered*)arg;
+  switch(body_type) {
+  case IMAP_BODY_TYPE_RFC822:
+    g_warning("Server sends unrequested RFC822 response");
+    break; /* This is really unexpected response! */
+  case IMAP_BODY_TYPE_HEADER:
+    phto->cb(seqno, str, len, phto->arg);
+    phto->wrote_header = TRUE;
+    if(phto->body) {
+    printf("Writing saved body\n");
+      phto->cb(seqno, phto->body, phto->length, phto->arg);
+      g_free(phto->body); phto->body = NULL;
+    }
+    break;
+  case IMAP_BODY_TYPE_TEXT:
+  case IMAP_BODY_TYPE_BODY:
+    if(phto->wrote_header) {
+      phto->cb(seqno, str, len, phto->arg);
+    } else {
+      printf("saving body\n");
+      phto->body = g_malloc(len);
+      phto->length = len;
+      memcpy(phto->body, str, len);
+    }
+    break;
+  }
+}
+
+
 ImapResponse
 imap_mbox_handle_fetch_rfc822_uid(ImapMboxHandle* handle, unsigned uid, 
-                              FILE *fl)
+                                  gboolean peek, FILE *fl)
 {
-  char cmd[40];
-  ImapFetchBodyCb cb = handle->body_cb;
+  char cmd[80];
+  ImapFetchBodyInternalCb cb = handle->body_cb;
   void          *arg = handle->body_arg;
   ImapResponse rc;
+  char *cmdstr;
+  struct FetchBodyHeaderText separate_arg;
+
 
   IMAP_REQUIRED_STATE1(handle, IMHS_SELECTED, IMR_BAD);
   HANDLE_LOCK(handle);
-  handle->body_cb  = write_nstring;
-  handle->body_arg = fl;
-  sprintf(cmd, "UID FETCH %u RFC822", uid);
+
+  /* Consider switching between BODY.PEEK[HEADER] BODY.PEEK[TEXT] and
+     BODY[HEADER] BODY[TEXT] - this would simplify the callback
+     code. */
+  if(peek) {
+    handle->body_cb  = write_header_text_ordered;
+    handle->body_arg = &separate_arg;
+    separate_arg.body = NULL;
+    separate_arg.wrote_header = FALSE;
+    separate_arg.out_file = fl;
+    cmdstr = "UID FETCH %u (BODY.PEEK[HEADER] BODY.PEEK[TEXT])";
+  } else {
+    handle->body_cb  = write_nstring;
+    handle->body_arg = fl;
+    cmdstr = "UID FETCH %u RFC822";
+  }
+
+  snprintf(cmd, sizeof(cmd), cmdstr, uid);
   rc = imap_cmd_exec(handle, cmd);
+  if(peek) {
+    g_free(separate_arg.body); /* This should never be needed. */
+  }
+
   handle->body_cb  = cb;
   handle->body_arg = arg;
   HANDLE_UNLOCK(handle);
@@ -980,20 +1225,18 @@ struct ImapBinaryData {
 };
 
 static void
-imap_binary_handler(unsigned seqno, const char *buf,
-                    int buflen, void* arg)
+imap_binary_handler(unsigned seqno, ImapFetchBodyType body_type,
+		    const char *buf, size_t buflen, void* arg)
 {
   struct ImapBinaryData *ibd = (struct ImapBinaryData*)arg;
   if(ibd->first_run) {
-    static const char binary_header[] =
-      "Content-Transfer-Encoding: binary\r\n\r\n";
     char *content_type = imap_body_get_content_type(ibd->body);
-    char *str = g_strdup_printf("Content-Type: %s\r\n", content_type);
+    char *str = g_strdup_printf("Content-Type: %s\r\n"
+				"Content-Transfer-Encoding: binary\r\n\r\n",
+				content_type);
     g_free(content_type);
     ibd->body_cb(seqno, str, strlen(str), ibd->body_arg);
     g_free(str);
-    ibd->body_cb(seqno, binary_header, sizeof(binary_header)-1,
-                 ibd->body_arg);
     ibd->first_run = FALSE;
   }
   ibd->body_cb(seqno, buf, buflen, ibd->body_arg);
@@ -1002,13 +1245,16 @@ imap_binary_handler(unsigned seqno, const char *buf,
 ImapResponse
 imap_mbox_handle_fetch_body(ImapMboxHandle* handle, 
                             unsigned seqno, const char *section,
+                            gboolean peek_only,
                             ImapFetchBodyOptions options,
                             ImapFetchBodyCb body_cb, void *arg)
 {
   char cmd[160];
-  ImapFetchBodyCb fcb = handle->body_cb;
+  ImapFetchBodyInternalCb fcb = handle->body_cb;
   void          *farg = handle->body_arg;
   ImapResponse rc;
+  const gchar *peek_string = peek_only ? ".PEEK" : "";
+  struct PassHeaderTextOrdered pass_ordered_data;
 
   IMAP_REQUIRED_STATE1(handle, IMHS_SELECTED, IMR_BAD);
 
@@ -1023,8 +1269,8 @@ imap_mbox_handle_fetch_body(ImapMboxHandle* handle,
     ibd.first_run = TRUE;
     handle->body_cb = imap_binary_handler;
     handle->body_arg = &ibd;
-    snprintf(cmd, sizeof(cmd), "FETCH %u BINARY[%s]",
-             seqno, section);
+    snprintf(cmd, sizeof(cmd), "FETCH %u BINARY%s[%s]",
+             seqno, peek_string, section);
     rc = imap_cmd_exec(handle, cmd);
     if(rc != IMR_NO) { /* unknown-cte */
       handle->body_cb  = fcb;
@@ -1032,12 +1278,17 @@ imap_mbox_handle_fetch_body(ImapMboxHandle* handle,
       return rc;
     }
   }
-  handle->body_cb  = body_cb;
-  handle->body_arg = arg;
+
+  handle->body_cb  = pass_header_text_ordered;
+  handle->body_arg = &pass_ordered_data;
+  pass_ordered_data.cb = body_cb;
+  pass_ordered_data.arg = arg;
+  pass_ordered_data.body = NULL;
+  pass_ordered_data.wrote_header = FALSE;
   /* Pure IMAP without extensions */
   if(options == IMFB_NONE)
-    snprintf(cmd, sizeof(cmd), "FETCH %u BODY[%s]",
-             seqno, section);
+    snprintf(cmd, sizeof(cmd), "FETCH %u BODY%s[%s]",
+             seqno, peek_string, section);
   else {
     char prefix[160];
     if(options == IMFB_HEADER) {
@@ -1054,10 +1305,11 @@ imap_mbox_handle_fetch_body(ImapMboxHandle* handle,
       prefix[sizeof(prefix)-1] = '\0';
     } else
       snprintf(prefix, sizeof(prefix), "%s.MIME", section);
-    snprintf(cmd, sizeof(cmd), "FETCH %u (BODY[%s] BODY[%s])",
-             seqno, prefix, section);
+    snprintf(cmd, sizeof(cmd), "FETCH %u (BODY%s[%s] BODY%s[%s])",
+             seqno, peek_string, prefix, peek_string, section);
   }
   rc = imap_cmd_exec(handle, cmd);
+  g_free(pass_ordered_data.body);
   handle->body_cb  = fcb;
   handle->body_arg = farg;
   return rc;
@@ -1095,7 +1347,7 @@ imap_store_prepare(ImapMboxHandle *h, unsigned msgcnt, unsigned*seqno,
   csd.handle = h; csd.msgcnt = msgcnt; csd.seqno = seqno;
   csd.flag = flg; csd.state = state;
   if(msgcnt == 0) return NULL;
-  seq = coalesce_seq_range(0, msgcnt-1, (CoalesceFunc)cf_flag, &csd);
+  seq = imap_coalesce_seq_range(0, msgcnt-1, (ImapCoalesceFunc)cf_flag, &csd);
   if(!seq) return NULL;
   str = enum_flag_to_str(flg);
   for(i=0; i<msgcnt; i++) {
@@ -1132,12 +1384,14 @@ imap_mbox_store_flag(ImapMboxHandle *h, unsigned msgcnt, unsigned*seqno,
   gchar* cmd;
 
   IMAP_REQUIRED_STATE1(h, IMHS_SELECTED, IMR_BAD);
+  HANDLE_LOCK(h);
   cmd = imap_store_prepare(h, msgcnt, seqno, flg, state);
   if(cmd) {
     res = imap_cmd_exec(h, cmd);
     g_free(cmd);
-    return res;
-  } else return IMR_OK;
+  } res = IMR_OK;
+  HANDLE_UNLOCK(h);
+  return res;
 }
 
 ImapResponse
@@ -1162,18 +1416,40 @@ imap_mbox_store_flag_a(ImapMboxHandle *h, unsigned msgcnt, unsigned*seqno,
     selected in handle to given mailbox on same server. */
 ImapResponse
 imap_mbox_handle_copy(ImapMboxHandle* handle, unsigned cnt, unsigned *seqno,
-                      const gchar *dest)
+                      const gchar *dest,
+		      ImapSequence *ret_sequence)
 {
+  ImapResponse rc;
   IMAP_REQUIRED_STATE1(handle, IMHS_SELECTED, IMR_BAD);
+  HANDLE_LOCK(handle);
   {
-  gchar *mbx7 = imap_utf8_to_mailbox(dest);
-  char *seq = 
-    coalesce_seq_range(0, cnt-1,(CoalesceFunc)simple_coealesce_func, seqno);
-  gchar *cmd = g_strdup_printf("COPY %s \"%s\"", seq, mbx7);
-  ImapResponse rc = imap_cmd_exec(handle, cmd);
-  g_free(seq); g_free(mbx7); g_free(cmd);
-  return rc;
+    gchar *mbx7 = imap_utf8_to_mailbox(dest);
+    char *seq = imap_coalesce_set(cnt, seqno);
+    gchar *cmd = g_strdup_printf("COPY %s \"%s\"", seq, mbx7);
+    unsigned cmdno;
+    gboolean use_uidplus = imap_mbox_handle_can_do(handle, IMCAP_UIDPLUS);
+
+    if(ret_sequence) {
+      ret_sequence->ranges = NULL;
+      handle->uidplus.store_response = 1;
+    } else
+      handle->uidplus.store_response = 0;
+
+    rc = imap_cmd_exec_cmdno(handle, cmd, &cmdno);
+    g_free(seq); g_free(mbx7); g_free(cmd);
+    if(use_uidplus && ret_sequence) {
+      if(rc == IMR_OK /* && cmdno == handle->uidplus.cmdno */ ) {
+	ret_sequence->uid_validity = handle->uidplus.dst_uid_validity;
+	ret_sequence->ranges = g_list_reverse(handle->uidplus.dst);
+      } else {
+	g_list_free(handle->uidplus.dst);
+      }
+      handle->uidplus.dst = NULL;
+      handle->uidplus.store_response = 0;
+    }
   }
+  HANDLE_UNLOCK(handle);
+  return rc;
 }
 
 /* 6.4.8 UID Command */
@@ -1297,8 +1573,7 @@ imap_mbox_sort_msgno_srv(ImapMboxHandle *handle, ImapSortKey key,
   /* seq can be pretty long and g_strdup_printf has a limit on
    * string length so we create the command string in two steps. */
   keystr = sort_code_to_string(key);
-  seq = coalesce_seq_range(0, cnt-1,(CoalesceFunc)simple_coealesce_func,
-                           msgno);
+  seq = imap_coalesce_set(cnt, msgno);
   cmd= g_strdup_printf("SORT (%s%s) UTF-8 ", 
                        ascending ? "" : "REVERSE ",
                        keystr);
@@ -1596,17 +1871,22 @@ make_msgno_table(ImapMboxHandle*handle, unsigned seqno, GHashTable *msgnos)
                       GUINT_TO_POINTER(seqno));
 }
 
+/** Appends to given hash the message numbers that match specified
+    filter. */
 ImapResponse
 imap_mbox_filter_msgnos(ImapMboxHandle *handle, ImapSearchKey *filter,
 			GHashTable *msgnos)
 {
-  return imap_search_exec(handle, FALSE, filter,
-                          (ImapSearchCb)make_msgno_table, msgnos);
+  ImapResponse res;
+  HANDLE_LOCK(handle);
+  res= imap_search_exec(handle, FALSE, filter,
+			(ImapSearchCb)make_msgno_table, msgnos);
+  HANDLE_UNLOCK(handle);
+  return res;
 }
 
-/* imap_mbox_complete_msgids:
-   finds which msgnos are missing from the supplied table and
-   completes them if needed.
+/** Helper function for imap_mbox_complete_msgids. Tells whether
+    msg-id for a specific message needs to be fetched.
 */
 static unsigned
 need_msgid(unsigned seqno, GPtrArray* msgids)
@@ -1615,13 +1895,21 @@ need_msgid(unsigned seqno, GPtrArray* msgids)
 }
 
 static void
-msgid_cb(unsigned seqno, const char *buf, int buflen, void* arg)
+msgid_cb(unsigned seqno, ImapFetchBodyType body_type,
+	 const char *buf, size_t buflen, void* arg)
 {
   GPtrArray *arr = (GPtrArray*)arg;
   g_return_if_fail(seqno>=1 && seqno<=arr->len);
   g_ptr_array_index(arr, seqno-1) = g_strdup(buf);
 }
 
+/** Fills in provided array with msg-ids, starting at specified message.
+ *  @returns IMAP response code (OK, NO, etc).
+ * @param h the connection handle.
+ * @param msgids is the array of message ids.
+ * @param first_seqno_to_fetch the number of the first message to be
+ * fetched.
+ */
 ImapResponse
 imap_mbox_complete_msgids(ImapMboxHandle *h,
 			  GPtrArray *msgids,
@@ -1629,12 +1917,12 @@ imap_mbox_complete_msgids(ImapMboxHandle *h,
 {
   gchar *seq, *cmd;
   ImapResponse rc = IMR_OK;
-  ImapFetchBodyCb cb;
+  ImapFetchBodyInternalCb cb;
   void *arg;
 
   IMAP_REQUIRED_STATE1(h, IMHS_SELECTED, IMR_BAD);
-  seq = coalesce_seq_range(first_seqno_to_fetch, msgids->len,
-			   (CoalesceFunc)need_msgid, msgids);
+  seq = imap_coalesce_seq_range(first_seqno_to_fetch, msgids->len,
+				(ImapCoalesceFunc)need_msgid, msgids);
   if(seq) {
     cmd = g_strdup_printf("FETCH %s BODY.PEEK[HEADER.FIELDS (message-id)]",
 			  seq);
