@@ -21,6 +21,7 @@
 
 #include <gmime/gmime.h>
 #include "imap-handle.h"
+#include "imap_search.h"
 
 /* Any-State */
 int imap_mbox_handle_can_do(ImapMboxHandle* handle, ImapCapability cap);
@@ -59,6 +60,20 @@ typedef size_t (*ImapAppendFunc)(char*, size_t, void*);
 ImapResponse imap_mbox_append(ImapMboxHandle *handle, const char *mbox,
                               ImapMsgFlags flags, size_t sz, 
                               ImapAppendFunc dump_cb,  void* arg);
+
+typedef enum {
+  IMA_STAGE_NEW_MSG,
+  IMA_STAGE_PASS_DATA
+} ImapAppendMultiStage;
+
+typedef size_t (*ImapAppendMultiFunc)(char*, size_t,
+				      ImapAppendMultiStage stage,
+				      ImapMsgFlags *flags, void*);
+ImapResponse imap_mbox_append_multi(ImapMboxHandle *handle,
+				    const char *mbox,
+				    ImapAppendMultiFunc dump_cb,
+				    void* arg,
+				    ImapSequence *uid_range);
 #ifdef USE_IMAP_APPEND_STR /* not used currently */
 ImapResponse imap_mbox_append_str(ImapMboxHandle *handle, const char *mbox,
                               ImapMsgFlags flags, size_t sz, char *txt);
@@ -72,12 +87,19 @@ ImapResponse imap_mbox_close(ImapMboxHandle *h);
 ImapResult imap_mbox_search(ImapMboxHandle *h, const char* query);
 ImapResponse imap_mbox_noop(ImapMboxHandle *r);
 ImapResponse imap_mbox_expunge(ImapMboxHandle* h);
+ImapResponse imap_mbox_expunge_a(ImapMboxHandle *h);
 
 ImapResponse imap_mbox_store_flag(ImapMboxHandle *r, unsigned cnt,
                                   unsigned *seqno, ImapMsgFlag flg,
                                   gboolean state);
-ImapResponse imap_mbox_handle_copy(ImapMboxHandle* handle, unsigned cnt,
-                                   unsigned *seqno, const gchar *dest);
+unsigned imap_mbox_store_flag_a(ImapMboxHandle *r, unsigned cnt,
+				unsigned *seqno, ImapMsgFlag flg,
+				gboolean state);
+
+ImapResponse imap_mbox_handle_copy(ImapMboxHandle* handle,
+				   unsigned cnt, unsigned *seqno,
+				   const gchar *dest,
+				   ImapSequence *ret_sequence);
 
 ImapResponse imap_mbox_find_unseen(ImapMboxHandle * h, unsigned *msgcnt,
 				   unsigned **msgs);
@@ -92,15 +114,22 @@ ImapResponse imap_mbox_handle_fetch_set(ImapMboxHandle* handle,
                                         unsigned *set, unsigned cnt,
                                         ImapFetchType ift);
 
-ImapResponse imap_mbox_handle_fetch_rfc822(ImapMboxHandle* handle,
-                                           unsigned seqno, FILE *fl);
-ImapResponse imap_mbox_handle_fetch_rfc822_uid(ImapMboxHandle* handle,
-                                               unsigned uid, FILE *fl);
 typedef void (*ImapFetchBodyCb)(unsigned seqno, const char *buf,
-				int buflen, void* arg);
+				size_t buflen, void* arg);
+
+ImapResponse imap_mbox_handle_fetch_rfc822(ImapMboxHandle* handle,
+					   unsigned cnt, unsigned *seqno, 
+                                           ImapFetchBodyCb cb,
+					   void *cb_data);
+
+ImapResponse imap_mbox_handle_fetch_rfc822_uid(ImapMboxHandle* handle,
+                                               unsigned uid, gboolean peek,
+                                               FILE *fl);
+
 ImapResponse imap_mbox_handle_fetch_body(ImapMboxHandle* handle, 
                                          unsigned seqno, 
                                          const char *section,
+                                         gboolean peek_only,
                                          ImapFetchBodyOptions options,
                                          ImapFetchBodyCb body_handler,
                                          void *arg);
