@@ -56,28 +56,30 @@ typedef char ImapCmdTag[7]; /* Imap command tag */
 typedef enum
 {
   IMCAP_IMAP4 = 0,
-  IMCAP_IMAP4REV1,
+  IMCAP_IMAP4REV1,              /* RFC 3501: Internet Message Access Protocol */
   IMCAP_STATUS,
-  IMCAP_ACL,			/* RFC 2086: IMAP4 ACL extension */
-  IMCAP_NAMESPACE,              /* RFC 2342: IMAP4 Namespace */
   IMCAP_AANONYMOUS, 	        /* RFC 2245: AUTH=ANONYMOUS */
   IMCAP_ACRAM_MD5,		/* RFC 2195: CRAM-MD5 authentication */
   IMCAP_AGSSAPI,		/* RFC 1731: GSSAPI authentication */
   IMCAP_APLAIN,                 /* RFC 2595: */
+  IMCAP_ACL,			/* RFC 2086: IMAP4 ACL extension */
+  IMCAP_BINARY,                 /* RFC 3516 */
+  IMCAP_CHILDREN,               /* RFC 3348 */
+  IMCAP_ESEARCH,                /* RFC 4731 */
+  IMCAP_IDLE,                   /* RFC 2177 */
+  IMCAP_LITERAL,                /* RFC 2088 */
+  IMCAP_LOGINDISABLED,		/* RFC 2595 */
+  IMCAP_MULTIAPPEND,            /* RFC 3502 */
+  IMCAP_NAMESPACE,              /* RFC 2342: IMAP4 Namespace */
+  IMCAP_SASLIR,                 /* imap-sasl-initial-response: SASL-IR */
+  IMCAP_SCAN,                   /* FIXME: RFC? */
   IMCAP_STARTTLS,		/* RFC 2595: STARTTLS */
-  IMCAP_LOGINDISABLED,		/*           LOGINDISABLED */
   IMCAP_SORT,                   /* SORT and THREAD described at: */
-  /* http://www.ietf.org/internet-drafts/draft-ietf-imapext-sort-13.txt */
   IMCAP_THREAD_ORDEREDSUBJECT,
   IMCAP_THREAD_REFERENCES,
-  IMCAP_UNSELECT,               /* FIXME: RFC? */
-  IMCAP_SCAN,                   /* FIXME: RFC? */
-  IMCAP_CHILDREN,               /* RFC 3348 */
-  IMCAP_LITERAL,                /* RFC 2088 */
-  IMCAP_IDLE,                   /* RFC 2177 */
-  IMCAP_SASLIR,                 /* imap-sasl-initial-response: SASL-IR */
-  IMCAP_BINARY,                 /* RFC 3516 */
-  IMCAP_ESEARCH,                /* RFC 4731 */
+  /* http://www.ietf.org/internet-drafts/draft-ietf-imapext-sort-13.txt */
+  IMCAP_UIDPLUS,                /* RFC 4315 */
+  IMCAP_UNSELECT,               /* RFC 3691 */
   IMCAP_FETCHBODY,              /* basic imap implemented correctly by
                                  * most imap servers but not all. We
                                  * have to detect that. */
@@ -121,7 +123,7 @@ void imap_handle_set_monitorcb(ImapMboxHandle* h, ImapMonitorCb cb, void*);
 void imap_handle_set_infocb(ImapMboxHandle* h, ImapInfoCb cb, void*);
 void imap_handle_set_usercb(ImapMboxHandle* h, ImapUserCb cb, void*);
 void imap_handle_set_flagscb(ImapMboxHandle* h, ImapFlagsCb cb, void*);
-void imap_handle_set_timeout(ImapMboxHandle *, int milliseconds);
+int imap_handle_set_timeout(ImapMboxHandle *, int milliseconds);
 gboolean imap_handle_idle_enable(ImapMboxHandle *, int seconds);
 gboolean imap_handle_idle_disable(ImapMboxHandle *);
 gboolean imap_handle_op_cancelled(ImapMboxHandle *h);
@@ -179,36 +181,26 @@ ImapBody *imap_message_get_body_from_section(ImapMessage *msg,
                                              const char *section);
 
 /* ================ END OF BODY STRUCTURE FUNCTIONS ==================== */
-/* ================ BEGIN OF SEARCH FUNCTIONS ========================== */
-typedef enum { IMSE_S_BCC, IMSE_S_BODY, IMSE_S_CC, IMSE_S_FROM, 
-               IMSE_S_SUBJECT, IMSE_S_TEXT, IMSE_S_TO, 
-               IMSE_S_HEADER } ImapSearchHeader;
 
-typedef enum { IMSE_D_BEFORE = -1, IMSE_D_ON = 0,
-               IMSE_D_SINCE  = 1 } ImapSearchDateRange;
+/** Stores data returned when UIDPLUS extension is provided. */
+typedef struct ImapSequence_ {
+  GList *ranges; /**< list of ImapUidRange items */
+  unsigned uid_validity;
+}  ImapSequence;
 
-typedef struct ImapSearchKey_ ImapSearchKey;
+/** defines range [lo, hi] */
+typedef struct ImapUidRange_ {
+  unsigned lo, hi; 
+} ImapUidRange;
 
-ImapSearchKey *imap_search_key_new_not(unsigned negated,
-                                       ImapSearchKey *list);
-ImapSearchKey *imap_search_key_new_or(unsigned negated,
-                                      ImapSearchKey *a, ImapSearchKey *b);
-ImapSearchKey *imap_search_key_new_flag(unsigned negated, ImapMsgFlag flg);
-ImapSearchKey *imap_search_key_new_string(unsigned negated,
-                                          ImapSearchHeader hdr,
-                                          const char *string, 
-                                          const char *user_header);
-ImapSearchKey* imap_search_key_new_size_greater(unsigned negated, size_t sz);
-ImapSearchKey* imap_search_key_new_date(ImapSearchDateRange range,
-                                        int internal, time_t tm);
-ImapSearchKey* imap_search_key_new_range(unsigned negated, int uid,
-                                         unsigned lo, unsigned hi);
-
-void imap_search_key_free(ImapSearchKey *s);
-void imap_search_key_set_next(ImapSearchKey *list, ImapSearchKey *next);
-ImapResponse imap_search_exec(ImapMboxHandle *h, gboolean uid, 
-                              ImapSearchKey *s, ImapSearchCb cb, void *cb_arg);
-/* ================ END OF SEARCH FUNCTIONS ============================ */
+#define imap_sequence_empty(i_seq) ( (i_seq)->ranges == NULL)
+unsigned imap_sequence_length(ImapSequence *i_seq);
+unsigned imap_sequence_nth(ImapSequence *i_seq, unsigned nth);
+void imap_sequence_foreach(ImapSequence *i_seq,
+			   void(*cb)(unsigned uid, void *arg), void *cb_arg);
+#define imap_sequence_init(i_seq) \
+  do { (i_seq)->ranges = NULL; (i_seq)->uid_validity = 0; }while(0)
+void imap_sequence_release(ImapSequence *i_seq);
 
 /* ================ BEGIN OF MBOX_VIEW FUNCTIONS ======================= */
 typedef struct _MboxView MboxView;
