@@ -26,6 +26,7 @@
 #if defined(BALSA_USE_THREADS)
 #include <pthread.h>
 #define HANDLE_LOCK(h)   pthread_mutex_lock(&h->mutex)
+#define HANDLE_TRYLOCK(h) pthread_mutex_trylock(&h->mutex)
 #define HANDLE_UNLOCK(h) pthread_mutex_unlock(&h->mutex)
 #else
 #define HANDLE_LOCK(h) 
@@ -34,6 +35,20 @@
 
 #include "imap-commands.h"
 
+typedef enum {
+  IMAP_BODY_TYPE_RFC822, /**< as fetched with RFC822 */
+  IMAP_BODY_TYPE_HEADER, /** header of the message: BODY[HEADER] */
+  IMAP_BODY_TYPE_TEXT,   /**< content of the message part: BODY[TEXT] */
+  IMAP_BODY_TYPE_BODY    /**< a part as fetched with BODY[x] */
+} ImapFetchBodyType;
+
+
+typedef void (*ImapFetchBodyInternalCb)(unsigned seqno,
+					ImapFetchBodyType body_type,
+					const char *buf,
+					size_t buflen, void* arg);
+
+
 struct _MboxView {
   unsigned *arr;
   unsigned allocated, entries;
@@ -76,6 +91,13 @@ struct _ImapMboxHandle {
                           * processing of current line is finished. */
   GNode *thread_root; /* deprecated! */
 
+  struct {
+    GList* src; /**< returned by COPY */
+    GList* dst; /**< returned by APPEND and COPY */
+    unsigned dst_uid_validity;
+    unsigned store_response:1;
+  } uidplus;
+
   /* BYE handling depends on the state */
   gboolean doing_logout;
   ImapInfoCb info_cb;
@@ -85,7 +107,7 @@ struct _ImapMboxHandle {
 
   ImapFlagsCb flags_cb;
   void *flags_arg;
-  ImapFetchBodyCb body_cb;
+  ImapFetchBodyInternalCb body_cb;
   void *body_arg;
 
   ImapMonitorCb monitor_cb;
@@ -144,7 +166,10 @@ extern const char* msg_flags[6];
 
 void imap_mbox_resize_cache(ImapMboxHandle *h, unsigned new_size);
 
-ImapResponse imap_cmd_exec(ImapMboxHandle* handle, const char* cmd);
+ImapResponse imap_cmd_exec_cmdno(ImapMboxHandle* handle, const char* cmd,
+				 unsigned *cmdno);
+#define imap_cmd_exec(h, c) imap_cmd_exec_cmdno((h),(c),NULL)
+
 ImapResponse imap_cmd_issue(ImapMboxHandle* handle, const char* cmd);
 char* imap_mbox_gets(ImapMboxHandle *h, char* buf, size_t sz);
 
@@ -167,6 +192,12 @@ void imap_mbox_handle_set_state(ImapMboxHandle *h,
 #define imap_mbox_handle_set_msg(h,s) \
   do{g_free((h)->last_msg); (h)->last_msg = g_strdup(s); }while(0)
 
+typedef unsigned (*ImapCoalesceFunc)(int, void*);
+gchar* imap_coalesce_seq_range(int lo, int hi,
+			       ImapCoalesceFunc fun, void *data);
+unsigned imap_coalesce_func_simple(int i, unsigned msgno[]);
+gchar *imap_coalesce_set(int cnt, unsigned *seqnos);
+
 /* even more private functions */
 int imap_cmd_start(ImapMboxHandle* handle, const char* cmd, unsigned* cmdno);
 ImapResponse imap_cmd_step(ImapMboxHandle* handle, unsigned cmdno);
