@@ -103,7 +103,8 @@ void libbalsa_set_spool(const gchar * spool);
 
 void libbalsa_show_message_source(LibBalsaMessage * msg,
                                   const gchar * font,
-                                  gboolean *escape_specials);
+                                  gboolean *escape_specials,
+                                  gint * width, gint * height);
 gchar *libbalsa_rot(const gchar * pass);
 
 gchar *libbalsa_guess_email_address(void);
@@ -154,16 +155,20 @@ typedef enum {
 #define LIBBALSA_PROGRESS_INIT LIBBALSA_PROGRESS_NO
 /* We will not use the progress bar if the number of increments is less
  * than LIBBALSA_PROGRESS_MIN_COUNT, and we will not update the fraction
- * if the time since the last updata is less than
- * LIBBALSA_PROGRESS_MIN_UPDATE seconds. */
-#define LIBBALSA_PROGRESS_MIN_COUNT  400
-#define LIBBALSA_PROGRESS_MIN_UPDATE 0.05
+ * if the time since the last update is less than
+ * LIBBALSA_PROGRESS_MIN_UPDATE_SECS seconds or if the fraction has
+ * increased by less than LIBBALSA_PROGRESS_MIN_UPDATE_STEP. */
+#define LIBBALSA_PROGRESS_MIN_COUNT        400
+#define LIBBALSA_PROGRESS_MIN_UPDATE_USECS 50000
+#define LIBBALSA_PROGRESS_MIN_UPDATE_STEP  0.05
 
 extern void (*libbalsa_progress_set_text) (LibBalsaProgress * progress,
                                            const gchar * text,
                                            guint total);
 extern void (*libbalsa_progress_set_fraction) (LibBalsaProgress * progress,
                                                gdouble fraction);
+extern void (*libbalsa_progress_set_activity) (gboolean set,
+                                               const gchar * text);
 
 /*
  * Face and X-Face header support.
