@@ -34,10 +34,7 @@
 #include "libbalsa-conf.h"
 #include "mailbox-filter.h"
 #include "misc.h"
-#include "i18n.h"
-
-/* FIXME : double definition : first is in save-restore.c */
-#define BALSA_CONFIG_PREFIX "balsa/"
+#include <glib/gi18n.h>
 
 /* Returns a slist of filters having the corresponding when field
  * There is no copy, the new list references object of the source list
@@ -55,50 +52,57 @@ libbalsa_mailbox_filters_when(GSList * filters, gint when)
     return lst;
 }
 
-/* Looks for a mailbox filters section with MBOX_URL field equals to mbox->url
- * returns the section name or NULL if none found
+/* Looks for a mailbox filters group with MBOX_URL field equals to mbox->url
+ * returns the group name or NULL if none found
  * The returned string has to be freed by the caller
  */
+struct lbmf_section_lookup_info {
+    const gchar *name;
+    gchar *group;
+};
 
-gchar*
+static gboolean
+lbmf_section_lookup_func(const gchar * key, const gchar * value,
+                         gpointer data)
+{
+    struct lbmf_section_lookup_info *info = data;
+    gchar *url;
+
+    libbalsa_conf_push_group(key);
+    url = libbalsa_conf_get_string(MAILBOX_FILTERS_URL_KEY);
+    libbalsa_conf_pop_group();
+    if (strcmp(url, info->name) == 0)
+        info->group = g_strdup(key);
+    g_free(url);
+
+    return info->group != NULL;
+}
+
+gchar *
 mailbox_filters_section_lookup(const gchar * name)
 {
-    gchar * key, *section = NULL;
-    void * iterator;
+    struct lbmf_section_lookup_info info;
 
-    g_return_val_if_fail(name && name[0],NULL);
-    iterator = libbalsa_conf_init_iterator_sections(BALSA_CONFIG_PREFIX);
-    while (!section &&
-	   (iterator = libbalsa_conf_iterator_next(iterator, &key, NULL))) {
-	if (libbalsa_str_has_prefix(key, MAILBOX_FILTERS_SECTION_PREFIX)) {
-	    gchar *url;
+    g_return_val_if_fail(name && name[0], NULL);
 
-	    section = g_strconcat(BALSA_CONFIG_PREFIX, key, "/", NULL);
-	    libbalsa_conf_push_prefix(section);
-	    url = libbalsa_conf_get_string(MAILBOX_FILTERS_URL_KEY);
-	    libbalsa_conf_pop_prefix();
-	    if (strcmp(url, name) != 0) {
-		g_free(section);
-		section = NULL;
-	    }
-	    g_free(url);
-	}
-	g_free(key);
-    }
-    g_free(iterator);
-    return section;
+    info.name = name;
+    info.group = NULL;
+    libbalsa_conf_foreach_group(MAILBOX_FILTERS_SECTION_PREFIX,
+                                  lbmf_section_lookup_func, &info);
+
+    return info.group;
 }
 
 void
 config_mailbox_filters_load(LibBalsaMailbox * mbox)
 {
-    gchar * section;
+    gchar * group;
 
-    section = mailbox_filters_section_lookup(mbox->url ? mbox->url : mbox->name);
-    if (section) {
-	libbalsa_conf_push_prefix(section);
-	g_free(section);
+    group = mailbox_filters_section_lookup(mbox->url ? mbox->url : mbox->name);
+    if (group) {
+	libbalsa_conf_push_group(group);
+	g_free(group);
 	libbalsa_mailbox_filters_load_config(mbox);
-	libbalsa_conf_pop_prefix();
+	libbalsa_conf_pop_group();
     }
 }
