@@ -72,7 +72,8 @@ static void libbalsa_mailbox_maildir_load_config(LibBalsaMailbox * mailbox,
 
 static GMimeStream *libbalsa_mailbox_maildir_get_message_stream(LibBalsaMailbox *
 							   mailbox,
-							   guint msgno);
+								guint msgno,
+								gboolean peek);
 
 static gboolean libbalsa_mailbox_maildir_open(LibBalsaMailbox * mailbox,
 					      GError **err);
@@ -86,11 +87,10 @@ static gboolean
 libbalsa_mailbox_maildir_fetch_message_structure(LibBalsaMailbox * mailbox,
 						 LibBalsaMessage * message,
 						 LibBalsaFetchFlag flags);
-static gboolean libbalsa_mailbox_maildir_add_message(LibBalsaMailbox *
-                                                     mailbox,
-                                                     GMimeStream * stream,
-                                                     LibBalsaMessageFlag
-                                                     flags, GError ** err);
+static guint libbalsa_mailbox_maildir_add_messages(LibBalsaMailbox *
+						   mailbox,
+						   LibBalsaAddMessageIterator m,
+						   void *m_arg, GError ** err);
 static guint
 libbalsa_mailbox_maildir_total_messages(LibBalsaMailbox * mailbox);
 
@@ -167,7 +167,8 @@ libbalsa_mailbox_maildir_class_init(LibBalsaMailboxMaildirClass * klass)
 	libbalsa_mailbox_maildir_close_mailbox;
     libbalsa_mailbox_class->fetch_message_structure =
 	libbalsa_mailbox_maildir_fetch_message_structure;
-    libbalsa_mailbox_class->add_message = libbalsa_mailbox_maildir_add_message;
+    libbalsa_mailbox_class->add_messages =
+	libbalsa_mailbox_maildir_add_messages;
     libbalsa_mailbox_class->total_messages =
 	libbalsa_mailbox_maildir_total_messages;
 
@@ -311,7 +312,7 @@ libbalsa_mailbox_maildir_load_config(LibBalsaMailbox * mailbox,
 
 static GMimeStream *
 libbalsa_mailbox_maildir_get_message_stream(LibBalsaMailbox * mailbox,
-					    guint msgno)
+					    guint msgno, gboolean peek)
 {
     struct message_info *msg_info;
 
@@ -464,6 +465,11 @@ lbm_maildir_parse_subdirs(LibBalsaMailboxMaildir * mdir)
     }
 
     lbm_maildir_parse(mdir, "cur", &fileno);
+    /* We parse "new" after "cur", so that any recent messages will have
+     * higher msgnos than any current messages. That ensures that the
+     * message tree saved by LibBalsaMailboxLocal is still valid, and
+     * that the new messages will be inserted correctly into the tree by
+     * libbalsa_mailbox_local_add_messages. */
     lbm_maildir_parse(mdir, "new", &fileno);
 }
 
@@ -815,6 +821,7 @@ libbalsa_mailbox_maildir_sync(LibBalsaMailbox * mailbox, gboolean expunge)
 
         /* Reparse, to get the fileno entries right. */
         lbm_maildir_parse_subdirs(mdir);
+        mailbox->msg_tree_changed = TRUE;
 
         if (stat(mdir->tmpdir, &st) == 0)
             libbalsa_mailbox_set_mtime(mailbox, st.st_mtime);
@@ -950,6 +957,27 @@ libbalsa_mailbox_maildir_add_message(LibBalsaMailbox * mailbox,
     return retval;
 }
 
+static guint
+libbalsa_mailbox_maildir_add_messages(LibBalsaMailbox * mailbox,
+				      LibBalsaAddMessageIterator msg_iterator,
+				      void *arg,
+				      GError **err)
+{
+    LibBalsaMessageFlag flag;
+    GMimeStream *stream;
+
+    guint cnt = 0;
+    while( msg_iterator(&flag, &stream, arg) ) {
+	gboolean success =
+	    libbalsa_mailbox_maildir_add_message(mailbox, stream, flag, err);
+	g_object_unref(stream);
+	if(!success)
+	    break;
+	cnt++;
+    }
+    return cnt;
+}
+
 static guint
 libbalsa_mailbox_maildir_total_messages(LibBalsaMailbox * mailbox)
 {
