@@ -65,10 +65,12 @@ static LibBalsaMailboxLocalClass *parent_class = NULL;
 
 static void libbalsa_mailbox_mbox_class_init(LibBalsaMailboxMboxClass *klass);
 static void libbalsa_mailbox_mbox_init(LibBalsaMailboxMbox * mailbox);
+static void libbalsa_mailbox_mbox_dispose(GObject * object);
 
 static GMimeStream *libbalsa_mailbox_mbox_get_message_stream(LibBalsaMailbox *
 							     mailbox,
-							     guint msgno);
+							     guint msgno,
+							     gboolean peek);
 static gint lbm_mbox_check_files(const gchar * path, gboolean create);
 static void libbalsa_mailbox_mbox_remove_files(LibBalsaMailboxLocal *mailbox);
 
@@ -88,11 +90,10 @@ static gboolean
 libbalsa_mailbox_mbox_fetch_message_structure(LibBalsaMailbox * mailbox,
                                               LibBalsaMessage * message,
                                               LibBalsaFetchFlag flags);
-static gboolean libbalsa_mailbox_mbox_add_message(LibBalsaMailbox *
-                                                  mailbox,
-                                                  GMimeStream * stream,
-                                                  LibBalsaMessageFlag
-                                                  flags, GError ** err);
+static guint libbalsa_mailbox_mbox_add_messages(LibBalsaMailbox *mailbox,
+						LibBalsaAddMessageIterator m,
+						void *m_arg,
+						GError ** err);
 static guint
 libbalsa_mailbox_mbox_total_messages(LibBalsaMailbox * mailbox);
 #if BALSA_USE_THREADS
@@ -163,7 +164,7 @@ libbalsa_mailbox_mbox_class_init(LibBalsaMailboxMboxClass * klass)
 	libbalsa_mailbox_mbox_close_mailbox;
     libbalsa_mailbox_class->fetch_message_structure =
 	libbalsa_mailbox_mbox_fetch_message_structure;
-    libbalsa_mailbox_class->add_message = libbalsa_mailbox_mbox_add_message;
+    libbalsa_mailbox_class->add_messages = libbalsa_mailbox_mbox_add_messages;
     libbalsa_mailbox_class->total_messages =
 	libbalsa_mailbox_mbox_total_messages;
 #if BALSA_USE_THREADS
@@ -175,6 +176,7 @@ libbalsa_mailbox_mbox_class_init(LibBalsaMailboxMboxClass * klass)
 	libbalsa_mailbox_mbox_remove_files;
 
     libbalsa_mailbox_local_class->get_info = lbm_mbox_get_info;
+    object_class->dispose = libbalsa_mailbox_mbox_dispose;
 }
 
 
@@ -184,6 +186,13 @@ libbalsa_mailbox_mbox_init(LibBalsaMailboxMbox * mbox)
 {
 }
 
+static void
+libbalsa_mailbox_mbox_dispose(GObject * object)
+{
+    if(MAILBOX_OPEN(LIBBALSA_MAILBOX(object)))
+	libbalsa_mailbox_mbox_close_mailbox(LIBBALSA_MAILBOX(object), FALSE);
+}
+
 static gint
 lbm_mbox_check_files(const gchar * path, gboolean create)
 {
@@ -273,7 +282,7 @@ message_info_from_msgno(LibBalsaMailboxMbox * mbox, guint msgno)
 
 static GMimeStream *
 libbalsa_mailbox_mbox_get_message_stream(LibBalsaMailbox * mailbox,
-                                         guint msgno)
+                                         guint msgno, gboolean peek)
 {
     LibBalsaMailboxMbox *mbox;
     struct message_info *msg_info;
@@ -332,9 +341,9 @@ static void mbox_unlock(LibBalsaMailbox * mailbox, GMimeStream *stream)
 static void
 lbm_mbox_header_cb(GMimeParser * parser, const char *header,
                    const char *value, off_t offset,
-                   struct message_info **msg_info_p)
+                   gpointer user_data)
 {
-    struct message_info *msg_info = *msg_info_p;
+    struct message_info *msg_info = *(struct message_info **) user_data;
 
     if (g_ascii_strcasecmp(header, "Status") == 0 && msg_info->status < 0)
         msg_info->status = offset;
@@ -461,7 +470,6 @@ parse_mailbox(LibBalsaMailboxMbox * mbox)
     g_mime_parser_set_respect_content_length(gmime_parser, TRUE);
     g_mime_parser_set_header_regex(gmime_parser,
                                    "^Status|^X-Status|^MIME-Version",
-				   (GMimeParserHeaderRegexFunc)
 				   lbm_mbox_header_cb, &msg_info_p);
 
     libbalsa_mailbox_local_set_threading_info(local);
@@ -1087,7 +1095,7 @@ lbm_mbox_get_mime_message(LibBalsaMailbox * mailbox,
     GMimeParser *parser;
     GMimeMessage *mime_message;
 
-    stream = libbalsa_mailbox_mbox_get_message_stream(mailbox, msgno);
+    stream = libbalsa_mailbox_mbox_get_message_stream(mailbox, msgno, TRUE);
     if (!stream)
 	return NULL;
     libbalsa_mime_stream_shared_lock(stream);
@@ -1609,7 +1617,6 @@ libbalsa_mailbox_mbox_sync(LibBalsaMailbox * mailbox, gboolean expunge)
     g_mime_parser_set_respect_content_length(gmime_parser, TRUE);
     g_mime_parser_set_header_regex(gmime_parser,
                                    "^Status|^X-Status|^MIME-Version",
-				   (GMimeParserHeaderRegexFunc)
 				   lbm_mbox_header_cb, &msg_info);
     for (j = first; j < mbox->msgno_2_msg_info->len; ) {
 	GMimeMessage *mime_msg;
@@ -1957,6 +1964,27 @@ libbalsa_mailbox_mbox_add_message(LibBalsaMailbox * mailbox,
     return retval >= 0;
 }
 
+static guint
+libbalsa_mailbox_mbox_add_messages(LibBalsaMailbox * mailbox,
+				   LibBalsaAddMessageIterator msg_iterator,
+				   void *arg,
+				   GError **err)
+{
+    LibBalsaMessageFlag flag;
+    GMimeStream *stream;
+
+    guint cnt = 0;
+    while( msg_iterator(&flag, &stream, arg) ) {
+	gboolean success =
+	    libbalsa_mailbox_mbox_add_message(mailbox, stream, flag, err);
+	g_object_unref(stream);
+	if(!success)
+	    break;
+	cnt++;
+    }
+    return cnt;
+}
+
 static guint
 libbalsa_mailbox_mbox_total_messages(LibBalsaMailbox * mailbox)
 {
