@@ -900,7 +900,7 @@ lb_message_headers_basic_from_gmime(LibBalsaMessageHeaders *headers,
 	 */
 	const GMimeContentType *content_type;
 	gchar *str;
-
+	g_return_if_fail(headers->content_type == NULL);
 	content_type = g_mime_object_get_content_type(mime_msg->mime_part);
 	str = g_mime_content_type_to_string(content_type);
 	headers->content_type = g_mime_content_type_new_from_string(str);
@@ -1112,28 +1112,33 @@ lbmsg_set_header(LibBalsaMessage *message, const gchar *name,
     if (g_ascii_strcasecmp(name, "Date") == 0) {
 	message->headers->date = g_mime_utils_header_decode_date(value, NULL);
     } else
-    if (g_ascii_strcasecmp(name, "From") == 0) {
+    if (message->headers->from == NULL &&
+	g_ascii_strcasecmp(name, "From") == 0) {
         message->headers->from = internet_address_parse_string(value);
     } else
-    if (g_ascii_strcasecmp(name, "To") == 0) {
+    if (message->headers->to_list == NULL &&
+        g_ascii_strcasecmp(name, "To") == 0) {
 	message->headers->to_list = internet_address_parse_string(value);
     } else
     if (g_ascii_strcasecmp(name, "In-Reply-To") == 0) {
 	libbalsa_message_set_in_reply_to_from_string(message, value);
     } else
-    if (g_ascii_strcasecmp(name, "Message-ID") == 0) {
+    if (message->message_id == NULL &&
+        g_ascii_strcasecmp(name, "Message-ID") == 0) {
 	message->message_id = g_mime_utils_decode_message_id(value);
     } else
     if (g_ascii_strcasecmp(name, "References") == 0) {
 	libbalsa_message_set_references_from_string(message, value);
     } else
-    if (g_ascii_strcasecmp(name, "Content-Type") == 0) {
+    if (message->headers->content_type == NULL &&
+	g_ascii_strcasecmp(name, "Content-Type") == 0) {
 	message->headers->content_type =
 	    g_mime_content_type_new_from_string(value);
     } else
-    if (g_ascii_strcasecmp(name, "Disposition-Notification-To") == 0) {
-        message->headers->dispnotify_to =
-            internet_address_parse_string(value);
+    if (message->headers->dispnotify_to == NULL &&
+	g_ascii_strcasecmp(name, "Disposition-Notification-To") == 0) {
+	message->headers->dispnotify_to =
+	    internet_address_parse_string(value);
     } else
 #ifdef MESSAGE_COPY_CONTENT
     if (g_ascii_strcasecmp(name, "Content-Length") == 0) {
@@ -1189,31 +1194,48 @@ void
 libbalsa_message_load_envelope_from_stream(LibBalsaMessage * message,
                                            GMimeStream *gmime_stream)
 {
+    GMimeStream *gmime_stream_filter;
+    GMimeFilter *gmime_filter_crlf;
     GMimeStream *gmime_stream_buffer;
     GByteArray *line;
     guchar lookahead;
-    gboolean ret = FALSE;
 
     libbalsa_mime_stream_shared_lock(gmime_stream);
-    gmime_stream_buffer =
-        g_mime_stream_buffer_new(gmime_stream,
-                                 GMIME_STREAM_BUFFER_BLOCK_READ);
 
+    /* CRLF-filter the message stream; we do not want '\r' in header
+     * fields, and finding the empty line that separates the body from
+     * the header is simpler if it has no '\r' in it. */
+    gmime_stream_filter =
+        g_mime_stream_filter_new_with_stream(gmime_stream);
+
+    gmime_filter_crlf =
+        g_mime_filter_crlf_new(GMIME_FILTER_CRLF_DECODE,
+                               GMIME_FILTER_CRLF_MODE_CRLF_ONLY);
+    g_mime_stream_filter_add(GMIME_STREAM_FILTER(gmime_stream_filter),
+                                                 gmime_filter_crlf);
+    g_object_unref(gmime_filter_crlf);
+
+    /* Buffer the message stream, so we can read it line by line. */
+    gmime_stream_buffer =
+        g_mime_stream_buffer_new(gmime_stream_filter,
+                                 GMIME_STREAM_BUFFER_BLOCK_READ);
+    g_object_unref(gmime_stream_filter);
+
+    /* Read header fields until either:
+     * - we find an empty line, or
+     * - end of file.
+     */
     line = g_byte_array_new();
     do {
 	g_mime_stream_buffer_readln(gmime_stream_buffer, line);
-	while (!g_mime_stream_eos(gmime_stream_buffer)
-	       && g_mime_stream_read(gmime_stream_buffer,
-		                     (char *) &lookahead, 1) == 1) {
-		if (lookahead == ' ' || lookahead == '\t') {
-		    g_byte_array_append(line, &lookahead, 1);
-		    g_mime_stream_buffer_readln(gmime_stream_buffer, line);
-		} else
-		    break;
+	while (g_mime_stream_read(gmime_stream_buffer,
+		                  (char *) &lookahead, 1) == 1 
+               && (lookahead == ' ' || lookahead == '\t')) {
+            g_byte_array_append(line, &lookahead, 1);
+            g_mime_stream_buffer_readln(gmime_stream_buffer, line);
 	}
 	if (line->len == 0 || line->data[line->len-1]!='\n') {
-	    /* read error */
-	    ret = FALSE;
+	    /* EOF or read error; in either case, message has no body. */
 	    break;
 	}
 	line->data[line->len-1]='\0'; /* terminate line by overwriting '\n' */
@@ -1222,19 +1244,19 @@ libbalsa_message_load_envelope_from_stream(LibBalsaMessage * message,
 						FALSE)) {
 	    /* Ignore error return caused by malformed header. */
 	}
-	if (lookahead == '\n') {/* end of headers */
-	    ret = TRUE;
+	if (lookahead == '\n') {/* end of header */
+            /* Message looks valid--set its length. */
+            message->length = g_mime_stream_length(gmime_stream);
 	    break;
 	}
 	line->len = 0;
 	g_byte_array_append(line, &lookahead, 1);
-    } while (!g_mime_stream_eos(gmime_stream_buffer));
-    if (ret)
-        message->length = g_mime_stream_length(gmime_stream);
+    } while (TRUE);
+    g_byte_array_free(line, TRUE);
+
     g_object_unref(gmime_stream_buffer);
     g_mime_stream_reset(gmime_stream);
     libbalsa_mime_stream_shared_unlock(gmime_stream);
-    g_byte_array_free(line, TRUE);
 }
 
 void
@@ -1263,7 +1285,7 @@ libbalsa_message_stream(LibBalsaMessage * message)
 
     if (mailbox)
         return libbalsa_mailbox_get_message_stream(mailbox,
-                                                   message->msgno);
+                                                   message->msgno, FALSE);
 
     mime_stream = g_mime_stream_mem_new();
     g_mime_object_write_to_stream(GMIME_OBJECT(message->mime_msg),
