@@ -288,12 +288,6 @@ gboolean libbalsa_message_body_ref(LibBalsaMessage * message, gboolean read,
                                    gboolean fetch_all_headers);
 void libbalsa_message_body_unref(LibBalsaMessage * message);
 
-gboolean libbalsa_message_postpone(LibBalsaMessage * message,
-				   LibBalsaMailbox * draftbox,
-				   LibBalsaMessage * reply_message,
-				   gchar ** extra_headers,
-				   gboolean flow);
-
 /*
  * misc message releated functions
  */
