@@ -36,9 +36,7 @@ static GString *process_mime_multipart(LibBalsaMessage * message,
                                        LibBalsaMessageBody * body,
 				       gchar * reply_prefix_str,
 				       gint llen, gboolean ignore_html,
-                                       gboolean flow,
-				       LibBalsaCharsetFunc charset_cb,
-				       gpointer charset_cb_data);
+                                       gboolean flow);
 
 /* process_mime_part:
    returns string representation of given message part.
@@ -48,8 +46,7 @@ static GString *process_mime_multipart(LibBalsaMessage * message,
 GString *
 process_mime_part(LibBalsaMessage * message, LibBalsaMessageBody * body,
 		  gchar * reply_prefix_str, gint llen, gboolean ignore_html,
-                  gboolean flow, LibBalsaCharsetFunc charset_cb,
-		  gpointer charset_cb_data)
+                  gboolean flow)
 {
     gchar *res = NULL;
     size_t allocated;
@@ -68,8 +65,7 @@ process_mime_part(LibBalsaMessage * message, LibBalsaMessageBody * body,
 	break;
     case LIBBALSA_MESSAGE_BODY_TYPE_MULTIPART:
         reply = process_mime_multipart(message, body, reply_prefix_str,
-                                       llen, ignore_html, flow,
-                                       charset_cb, charset_cb_data);
+                                       llen, ignore_html, flow);
 	break;
     case LIBBALSA_MESSAGE_BODY_TYPE_TEXT:
 	/* don't return text/html stuff... */
@@ -91,30 +87,27 @@ process_mime_part(LibBalsaMessage * message, LibBalsaMessageBody * body,
 	}
 #endif /* HAVE_GTKHTML */
 
-        if (charset_cb)
-            charset_cb(libbalsa_message_body_charset(body),
-                       charset_cb_data);
+        if (flow && libbalsa_message_body_is_flowed(body)) {
+            /* we're making a `format=flowed' message, and the
+             * message we're quoting was flowed
+             *
+             * we'll assume it's going to the screen */
+            gboolean delsp = libbalsa_message_body_is_delsp(body);
 
-	if (llen > 0) {
-            if (flow && libbalsa_message_body_is_flowed(body)) {
-                /* we're making a `format=flowed' message, and the
-                 * message we're quoting was flowed
-                 *
-                 * we'll assume it's going to the screen */
-		gboolean delsp = libbalsa_message_body_is_delsp(body);
+            reply =
+                libbalsa_process_text_rfc2646(res, G_MAXINT, FALSE, TRUE,
+                                              reply_prefix_str != NULL,
+                                              delsp);
+            g_free(res);
+            break;
+        }
 
-		reply =
-		    libbalsa_process_text_rfc2646(res, G_MAXINT, FALSE,
-						  TRUE,
-						  reply_prefix_str != NULL,
-						  delsp);
-                g_free(res);
-                break;
-            }
+        if (llen > 0) {
 	    if (reply_prefix_str)
 		llen -= strlen(reply_prefix_str);
 	    libbalsa_wrap_string(res, llen);
 	}
+
         if (reply_prefix_str || flow) {
 	    gchar *str, *ptr;
 	    /* prepend the prefix to all the lines */
@@ -153,17 +146,14 @@ static GString *
 process_mime_multipart(LibBalsaMessage * message,
                        LibBalsaMessageBody * body,
 		       gchar * reply_prefix_str, gint llen,
-		       gboolean ignore_html, gboolean flow,
-		       LibBalsaCharsetFunc charset_cb,
-		       gpointer charset_cb_data)
+		       gboolean ignore_html, gboolean flow)
 {
     LibBalsaMessageBody *part;
     GString *res = NULL, *s;
 
     for (part = body->parts; part; part = part->next) {
         s = process_mime_part(message, part, reply_prefix_str, llen,
-                              ignore_html, flow, charset_cb,
-                              charset_cb_data);
+                              ignore_html, flow);
 	if (!s)
 	    continue;
 	if (res) {
@@ -180,8 +170,7 @@ process_mime_multipart(LibBalsaMessage * message,
 
 GString *
 content2reply(LibBalsaMessageBody * root, gchar * reply_prefix_str,
-	      gint llen, gboolean ignore_html, gboolean flow,
-	      LibBalsaCharsetFunc charset_cb, gpointer charset_cb_data)
+	      gint llen, gboolean ignore_html, gboolean flow)
 {
     LibBalsaMessage *message = root->message;
     LibBalsaMessageBody *body;
@@ -190,8 +179,7 @@ content2reply(LibBalsaMessageBody * root, gchar * reply_prefix_str,
     libbalsa_message_body_ref(message, FALSE, FALSE);
     for (body = root; body; body = body->next) {
 	res = process_mime_part(message, body, reply_prefix_str, llen,
-                                ignore_html, flow, charset_cb, 
-				charset_cb_data);
+                                ignore_html, flow);
 	if (!res)
 	    continue;
 	if (reply) {
@@ -683,7 +671,11 @@ is_in_url(GtkTextIter * iter, gint offset, GtkTextTag * url_tag)
 static gboolean prescanner(const gchar * p);
 static void mark_urls(GtkTextBuffer * buffer, GtkTextIter * iter,
                       GtkTextTag * tag, const gchar * p);
+#if USE_GREGEX
+static GRegex *get_url_reg(void);
+#else                           /* USE_GREGEX */
 static regex_t *get_url_reg(void);
+#endif                          /* USE_GREGEX */
 
 void
 libbalsa_unwrap_buffer(GtkTextBuffer * buffer, GtkTextIter * iter,
@@ -791,6 +783,31 @@ mark_urls(GtkTextBuffer * buffer, GtkTextIter * iter, GtkTextTag * tag,
           const gchar * line)
 {
     const gchar *p = line;
+#if USE_GREGEX
+    GRegex *url_reg = get_url_reg();
+    GMatchInfo *url_match;
+    GtkTextIter start = *iter;
+    GtkTextIter end = *iter;
+
+    while (g_regex_match(url_reg, p, 0, &url_match)) {
+        gint start_pos, end_pos;
+
+        if (g_match_info_fetch_pos(url_match, 0, &start_pos, &end_pos)) {
+            glong offset = g_utf8_pointer_to_offset(line, p + start_pos);
+            gtk_text_iter_set_line_offset(&start, offset);
+            offset = g_utf8_pointer_to_offset(line, p + end_pos);
+            gtk_text_iter_set_line_offset(&end, offset);
+            gtk_text_buffer_apply_tag(buffer, tag, &start, &end);
+
+            p += end_pos;
+        }
+        if (prescanner(p))
+            g_match_info_free(url_match);
+        else
+            break;
+    }
+    g_match_info_free(url_match);
+#else                          /* USE_GREGEX */
     regex_t *url_reg = get_url_reg();
     regmatch_t url_match;
     GtkTextIter start = *iter;
@@ -807,6 +824,7 @@ mark_urls(GtkTextBuffer * buffer, GtkTextIter * iter, GtkTextTag * tag,
         if (!prescanner(p))
             break;
     }
+#endif                          /* USE_GREGEX */
 }
 
 /* Prepare the buffer for sending with DelSp=Yes. */
@@ -881,6 +899,70 @@ prescanner(const gchar * s)
     return FALSE;
 }
 
+#if USE_GREGEX
+struct url_regex_info {
+    GRegex *url_reg;
+    const gchar *str;
+    const gchar *func;
+    const gchar *msg;
+};
+
+static GRegex *
+get_url_helper(struct url_regex_info *info)
+{
+    if (!info->url_reg) {
+        GError *err = NULL;
+
+        info->url_reg = g_regex_new(info->str, G_REGEX_CASELESS, 0, &err);
+        if (err) {
+            g_warning("%s %s: %s", info->func, info->msg, err->message);
+            g_error_free(err);
+        }
+    }
+
+    return info->url_reg;
+}
+
+static GRegex *
+get_url_reg(void)
+{
+    static struct url_regex_info info = {
+        NULL,
+        "(((https?|ftps?|nntp)://)|(mailto:|news:))"
+            "(%[0-9A-F]{2}|[-_.!~*';/?:@&=+$,#[:alnum:]])+",
+        __func__,
+        "url regex compilation failed"
+    };
+
+    return get_url_helper(&info);
+}
+
+static GRegex *
+get_ml_url_reg(void)
+{
+    static struct url_regex_info info = {
+        NULL,
+        "(%[0-9A-F]{2}|[-_.!~*';/?:@&=+$,#[:alnum:]]|[ \t]*[\r\n]+[ \t]*)+>",
+        __func__,
+        "multiline url regex compilation failed"
+    };
+
+    return get_url_helper(&info);
+}
+
+static GRegex *
+get_ml_flowed_url_reg(void)
+{
+    static struct url_regex_info info = {
+        NULL,
+        "(%[0-9A-F]{2}|[-_.!~*';/?:@&=+$,#[:alnum:]]|[ \t]+)+>",
+        __func__,
+        "multiline url regex compilation failed"
+    };
+
+    return get_url_helper(&info);
+}
+#else                           /* USE_GREGEX */
 static regex_t *
 get_url_reg(void)
 {
@@ -938,6 +1020,7 @@ get_ml_flowed_url_reg(void)
     
     return url_reg;
 }
+#endif                          /* USE_GREGEX */
 
 gboolean
 libbalsa_insert_with_url(GtkTextBuffer * buffer,
@@ -988,68 +1071,84 @@ libbalsa_insert_with_url(GtkTextBuffer * buffer,
 
 	if (prescanner(p)) {
 	    gint offset = 0;
-	    regex_t *url_reg = get_url_reg();
-	    regmatch_t url_match;
-	    gint match = regexec(url_reg, p, 1, &url_match, 0);
+#if USE_GREGEX
+            GRegex *url_reg = get_url_reg();
+            GMatchInfo *url_match;
+            gboolean match = g_regex_match(url_reg, p, 0, &url_match);
 
-	    while (!match) {
+	    while (match) {
 		gchar *buf;
 		gchar *spc;
+                gint start_pos, end_pos;
 
-		if (url_match.rm_so) {
+                if (!g_match_info_fetch_pos(url_match, 0, &start_pos, &end_pos))
+                    break;
+
+		if (start_pos > 0) {
 		    /* check if we hit a multi-line URL... (see RFC 1738) */
-		    if (all_p && (p[url_match.rm_so - 1] == '<' ||
-				  (url_match.rm_so > 4 &&
-				   !g_ascii_strncasecmp(p + url_match.rm_so - 5, "<URL:", 5)))) {
+		    if (all_p && (p[start_pos - 1] == '<' ||
+				  (start_pos > 4 &&
+				   !g_ascii_strncasecmp(p + start_pos - 5, "<URL:", 5)))) {
 			/* if the input is flowed, we will see a space at
 			 * url_match.rm_eo - in this case the complete remainder
 			 * of the ml uri should be in the passed buffer... */
 			if (url_info && url_info->buffer_is_flowed &&
-			    *(p + url_match.rm_eo) == ' ') {
-			    regex_t *ml_flowed_url_reg = get_ml_flowed_url_reg();
-			    regmatch_t ml_url_match;
+			    *(p + end_pos) == ' ') {
+                            GRegex *ml_flowed_url_reg = get_ml_flowed_url_reg();
+                            GMatchInfo *ml_url_match;
+                            gint ml_start_pos, ml_end_pos;
 
-			    if (!regexec(ml_flowed_url_reg,
-					 all_chars + offset + url_match.rm_eo, 1,
-					 &ml_url_match, 0) && ml_url_match.rm_so == 0)
-				url_match.rm_eo += ml_url_match.rm_eo - 1;
-			} else if (!strchr(p + url_match.rm_eo, '>')) {
-			    regex_t *ml_url_reg = get_ml_url_reg();
-			    regmatch_t ml_url_match;
-		    
-			    if (!regexec(ml_url_reg,
-					 all_chars + offset + url_match.rm_eo, 1,
-					 &ml_url_match, 0) && ml_url_match.rm_so == 0) {
+                            if (g_regex_match(ml_flowed_url_reg,
+                                              all_chars + offset + end_pos,
+                                              0, &ml_url_match)
+                                && g_match_info_fetch_pos(ml_url_match, 0,
+                                                          &ml_start_pos,
+                                                          &ml_end_pos)
+                                && ml_start_pos == 0)
+                                end_pos += ml_end_pos - 1;
+                            g_match_info_free(ml_url_match);
+			} else if (!strchr(p + end_pos, '>')) {
+                            gint ml_end_pos;
+                            GRegex *ml_url_reg = get_ml_url_reg();
+                            GMatchInfo *ml_url_match;
+
+                            if (g_regex_match(ml_url_reg,
+                                              all_chars + offset + end_pos,
+                                              0, &ml_url_match)
+                                && g_match_info_fetch_pos(ml_url_match, 0,
+                                                          NULL,
+                                                          &ml_end_pos)) {
 				GString *ml_url = g_string_new("");
-				const gchar *ml_p = all_chars + offset + url_match.rm_so;
-				gint ml_cnt =
-				    url_match.rm_eo - url_match.rm_so + ml_url_match.rm_eo - 1;
+				const gchar *ml_p =
+                                    all_chars + offset + start_pos;
+				gint ml_cnt;
 
+				ml_cnt = end_pos - start_pos + ml_end_pos - 1;
 				for (; ml_cnt; (ml_p++, ml_cnt--))
 				    if (*ml_p > ' ')
 					ml_url = g_string_append_c(ml_url, *ml_p);
 				url_info->ml_url = ml_url->str;
 				g_string_free(ml_url, FALSE);
 			    }
+                            g_match_info_free(ml_url_match);
 			}
 		    }
 
-		    buf = g_strndup(p, url_match.rm_so);
+		    buf = g_strndup(p, start_pos);
 		    gtk_text_buffer_insert_with_tags(buffer, &iter,
 						     buf, -1, tag, NULL);
 		    g_free(buf);
 		}
 
 		if (url_info->ml_url) {
-		    url_info->ml_url_buffer = g_string_new(p + url_match.rm_so);
+		    url_info->ml_url_buffer = g_string_new(p + start_pos);
 		    url_info->ml_url_buffer =
 			g_string_append_c(url_info->ml_url_buffer, '\n');
 		    return TRUE;
 		}
 
 		/* add the url - it /may/ contain spaces if the text is flowed */
-		buf = g_strndup(p + url_match.rm_so,
-				url_match.rm_eo - url_match.rm_so);
+		buf = g_strndup(p + start_pos, end_pos - start_pos);
 		if ((spc = strchr(buf, ' '))) {
 		    GString *uri_real = g_string_new("");
 		    gchar * p = buf;
@@ -1076,13 +1175,120 @@ libbalsa_insert_with_url(GtkTextBuffer * buffer,
 		}
 		g_free(buf);
 
-		p += url_match.rm_eo;
-		offset += url_match.rm_eo;
-		if (prescanner(p))
-		    match = regexec(url_reg, p, 1, &url_match, 0);
-		else
-		    match = -1;
+		p += end_pos;
+		offset += end_pos;
+		if (prescanner(p)) {
+                    g_match_info_free(url_match);
+                    match = g_regex_match(url_reg, p, 0, &url_match);
+                } else
+		    match = FALSE;
 	    }
+            g_match_info_free(url_match);
+#else                           /* USE_GREGEX */
+	    regex_t *url_reg = get_url_reg();
+	    regmatch_t url_match;
+	    gboolean match = regexec(url_reg, p, 1, &url_match, 0) == 0;
+
+	    while (match) {
+		gchar *buf;
+		gchar *spc;
+                gint start_pos, end_pos;
+
+                start_pos = url_match.rm_so;
+                end_pos   = url_match.rm_eo;
+
+		if (start_pos > 0) {
+		    /* check if we hit a multi-line URL... (see RFC 1738) */
+		    if (all_p && (p[start_pos - 1] == '<' ||
+				  (start_pos > 4 &&
+				   !g_ascii_strncasecmp(p + start_pos - 5, "<URL:", 5)))) {
+			/* if the input is flowed, we will see a space at
+			 * url_match.rm_eo - in this case the complete remainder
+			 * of the ml uri should be in the passed buffer... */
+			if (url_info && url_info->buffer_is_flowed &&
+			    *(p + end_pos) == ' ') {
+			    regex_t *ml_flowed_url_reg = get_ml_flowed_url_reg();
+			    regmatch_t ml_url_match;
+
+			    if (!regexec(ml_flowed_url_reg,
+					 all_chars + offset + end_pos, 1,
+					 &ml_url_match, 0)
+                                && ml_url_match.rm_so == 0)
+				end_pos += ml_url_match.rm_eo - 1;
+			} else if (!strchr(p + end_pos, '>')) {
+                            gint ml_end_pos;
+			    regex_t *ml_url_reg = get_ml_url_reg();
+			    regmatch_t ml_url_match;
+		    
+			    if (!regexec(ml_url_reg,
+					 all_chars + offset + end_pos, 1,
+					 &ml_url_match, 0)
+                                && ml_url_match.rm_so == 0) {
+				GString *ml_url = g_string_new("");
+				const gchar *ml_p =
+                                    all_chars + offset + start_pos;
+				gint ml_cnt;
+
+                                ml_end_pos = ml_url_match.rm_eo;
+				ml_cnt = end_pos - start_pos + ml_end_pos - 1;
+				for (; ml_cnt; (ml_p++, ml_cnt--))
+				    if (*ml_p > ' ')
+					ml_url = g_string_append_c(ml_url, *ml_p);
+				url_info->ml_url = ml_url->str;
+				g_string_free(ml_url, FALSE);
+			    }
+			}
+		    }
+
+		    buf = g_strndup(p, start_pos);
+		    gtk_text_buffer_insert_with_tags(buffer, &iter,
+						     buf, -1, tag, NULL);
+		    g_free(buf);
+		}
+
+		if (url_info->ml_url) {
+		    url_info->ml_url_buffer = g_string_new(p + start_pos);
+		    url_info->ml_url_buffer =
+			g_string_append_c(url_info->ml_url_buffer, '\n');
+		    return TRUE;
+		}
+
+		/* add the url - it /may/ contain spaces if the text is flowed */
+		buf = g_strndup(p + start_pos, end_pos - start_pos);
+		if ((spc = strchr(buf, ' '))) {
+		    GString *uri_real = g_string_new("");
+		    gchar * p = buf;
+
+		    while (spc) {
+			*spc = '\n';
+			g_string_append_len(uri_real, p, spc - p);
+			p = spc + 1;
+			spc = strchr(p, ' ');
+		    }
+		    g_string_append(uri_real, p);
+		    gtk_text_buffer_insert_with_tags(buffer, &iter, buf, -1,
+						     url_tag, tag, NULL);
+		    if (url_info->callback)
+			url_info->callback(buffer, &iter, uri_real->str, url_info->callback_data);
+		    g_string_free(uri_real, TRUE);
+		} else {
+		    gtk_text_buffer_insert_with_tags(buffer, &iter, buf, -1,
+						     url_tag, tag, NULL);
+
+		    /* remember the URL and its position within the text */
+		    if (url_info->callback)
+			url_info->callback(buffer, &iter, buf, url_info->callback_data);
+		}
+		g_free(buf);
+
+		p += end_pos;
+		offset += end_pos;
+		if (prescanner(p))
+		    match = regexec(url_reg, p, 1, &url_match, 0) == 0;
+		else
+		    match = FALSE;
+	    }
+#endif                          /* USE_GREGEX */
 	}
     }
 
@@ -1094,7 +1300,11 @@ libbalsa_insert_with_url(GtkTextBuffer * buffer,
 }
 
 void
+#if USE_GREGEX
+libbalsa_unwrap_selection(GtkTextBuffer * buffer, GRegex * rex)
+#else                           /* USE_GREGEX */
 libbalsa_unwrap_selection(GtkTextBuffer * buffer, regex_t * rex)
+#endif                          /* USE_GREGEX */
 {
     GtkTextIter start, end;
     gchar *line;
@@ -1165,6 +1375,34 @@ libbalsa_unwrap_selection(GtkTextBuffer * buffer, regex_t * rex)
     }
 }
 
+#if USE_GREGEX
+gboolean
+libbalsa_match_regex(const gchar * line, GRegex * rex, guint * count,
+                     guint * index)
+{
+    GMatchInfo *rm;
+    gint c;
+    const gchar *p;
+    gint end_pos;
+
+    c = 0;
+    for (p = line;
+         g_regex_match(rex, p, 0, &rm)
+         && g_match_info_fetch_pos(rm, 0, NULL, &end_pos)
+         && end_pos > 0;
+         p += end_pos) {
+        c++;
+        g_match_info_free(rm);
+    }
+    g_match_info_free(rm);
+
+    if (count)
+        *count = c;
+    if (index)
+        *index = p - line;
+    return c > 0;
+}
+#else                           /* USE_GREGEX */
 gboolean
 libbalsa_match_regex(const gchar * line, regex_t * rex, guint * count,
 		     guint * index)
@@ -1182,4 +1420,172 @@ libbalsa_match_regex(const gchar * line, regex_t * rex, guint * count,
 	*index = p - line;
     return c > 0;
 }
+#endif                          /* USE_GREGEX */
 
+
+GString *
+libbalsa_html_encode_hyperlinks(GString * paragraph)
+{
+    GString * retval;
+    gchar * p;
+#if USE_GREGEX
+    GRegex *url_reg = get_url_reg();
+    GMatchInfo *url_match;
+#else
+    regex_t *url_reg = get_url_reg();
+    regmatch_t url_match;
+#endif
+    gboolean match;
+    gchar * markup;
+
+    /* check for any url */
+    if (!prescanner(paragraph->str)) {
+        markup = g_markup_escape_text(paragraph->str, -1);
+        g_string_assign(paragraph, markup);
+        g_free(markup);
+        return paragraph;
+    }
+
+    /* got some url's... */
+    retval = g_string_new("");
+    p = paragraph->str;
+
+#if USE_GREGEX
+    match = g_regex_match(url_reg, p, 0, &url_match);
+#else
+    match = regexec(url_reg, p, 1, &url_match, 0) == 0;
+#endif
+
+    while (match) {
+        gint start_pos, end_pos;
+
+#if USE_GREGEX
+        if (!g_match_info_fetch_pos(url_match, 0, &start_pos, &end_pos))
+            break;
+#else
+        start_pos = url_match.rm_so;
+        end_pos   = url_match.rm_eo;
+#endif
+
+        /* add the url to the result */
+        if (start_pos > 0) {
+            markup = g_markup_escape_text(p, start_pos);
+            retval = g_string_append(retval, markup);
+            g_free(markup);
+        }
+        retval = g_string_append(retval, "<a href=\"");
+        retval = g_string_append_len(retval, p + start_pos, end_pos - start_pos);
+        retval = g_string_append(retval, "\">");
+        retval = g_string_append_len(retval, p + start_pos, end_pos - start_pos);
+        retval = g_string_append(retval, "</a>");
+
+        /* find next (if any) */
+        p += end_pos;
+        if (prescanner(p)) {
+#if USE_GREGEX
+            g_match_info_free(url_match);
+            match = g_regex_match(url_reg, p, 0, &url_match);
+#else
+            match = regexec(url_reg, p, 1, &url_match, 0) == 0;
+#endif
+        } else
+            match = FALSE;
+    }
+#if USE_GREGEX
+    g_match_info_free(url_match);
+#endif                          /* USE_GREGEX */
+
+    /* copy remainder */
+    if (*p != '\0') {
+        markup = g_markup_escape_text(p, -1);
+        retval = g_string_append(retval, markup);
+        g_free(markup);
+    }
+
+    /* done - free original, return new */
+    g_string_free(paragraph, TRUE);
+    return retval;
+}
+
+
+gchar *
+libbalsa_text_to_html(const gchar * title, const gchar * body, const gchar * lang)
+{
+    GString * html_body =
+        g_string_new("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">\n");
+    gchar * html_subject;
+    const gchar * start = body;
+    gchar * html_lang;
+    gchar * p;
+
+    /* set the html header, including the primary language and the title */
+    if (lang) {
+        html_lang = g_strdup(lang);
+        if ((p = strchr(html_lang, '_')))
+            *p = '-';
+    } else
+        html_lang = g_strdup("x-unknown");
+    html_subject = g_markup_escape_text(title, -1);
+    g_string_append_printf(html_body, 
+                           "<html lang=\"%s\"><head>\n"
+                           "<title>%s</title>\n"
+                           "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n"
+                           "<style type=\"text/css\">\n"
+                           "  p { margin-top: 0px; margin-bottom: 0px; }\n"
+                           "</style></head>\n"
+                           "<body>\n", html_lang, html_subject);
+    g_free(html_subject);
+    g_free(html_lang);
+
+    /* add the lines of the message body */
+    while (*start) {
+        const gchar * eol = strchr(start, '\n');
+        const gchar * p = start;
+        PangoDirection direction = PANGO_DIRECTION_NEUTRAL;
+        GString * html;
+        gsize idx;
+
+        if (!eol)
+            eol = start + strlen(start);
+
+        /* find the first real char to determine the paragraph direction */
+        while (p < eol && direction == PANGO_DIRECTION_NEUTRAL) {
+            direction = pango_unichar_direction(g_utf8_get_char(p));
+            p = g_utf8_next_char(p);
+        }
+
+        /* html escape the line */
+        html = g_string_new_len(start, eol - start);
+
+        /* encode hyperlinks */
+        html = libbalsa_html_encode_hyperlinks(html);
+
+        /* replace a series of n spaces by (n - 1) &nbsp; and one space */
+        idx = 0;
+        while (idx < html->len) {
+            if (html->str[idx] == ' ' && (idx == 0 || html->str[idx + 1] == ' ')) {
+                html->str[idx++] = '&';
+                html = g_string_insert(html, idx, "nbsp;");
+                idx += 5;
+            } else
+                idx = g_utf8_next_char(html->str + idx) - html->str;
+        }
+
+        /* append the paragraph, always stating the proper direction */
+        g_string_append_printf(html_body, "<p dir=\"%s\">%s</p>\n",
+                               direction == PANGO_DIRECTION_RTL ? "rtl" : "ltr",
+                               *html->str ? html->str : "&nbsp;");
+        g_string_free(html, TRUE);
+
+        /* next line */
+        start = eol;
+        if (*start)
+            start++;
+    }
+
+    /* close the html context */
+    html_body = g_string_append(html_body, "</body></html>\n");
+    
+    /* return the utf-8 encoded text/html */
+    return g_string_free(html_body, FALSE);
+}
