@@ -31,12 +31,14 @@
 #include <auth-client.h>
 #endif
 
-#ifdef HAVE_PCRE
-#  include <pcreposix.h>
-#else
-#  include <sys/types.h>
-#  include <regex.h>
-#endif
+#if !USE_GREGEX
+#  ifdef HAVE_PCRE
+#    include <pcreposix.h>
+#  else
+#    include <sys/types.h>
+#    include <regex.h>
+#  endif
+#endif                          /* USE_GREGEX */
 
 typedef enum _LibBalsaCodeset LibBalsaCodeset;
 
@@ -89,17 +91,17 @@ typedef struct _LibBalsaUrlInsertInfo LibBalsaUrlInsertInfo;
 struct _LibBalsaUrlInsertInfo {
     libbalsa_url_cb_t callback;
     gpointer callback_data;
+    gboolean buffer_is_flowed;
     gchar *ml_url;
     GString *ml_url_buffer;
 };
 
+#define LIBBALSA_ERROR_QUARK (g_quark_from_static_string("libbalsa"))
+
 extern LibBalsaCodesetInfo libbalsa_codeset_info[];
 GtkWidget *libbalsa_charset_button_new(void);
 LibBalsaTextAttribute libbalsa_text_attr_string(const gchar * string);
 LibBalsaTextAttribute libbalsa_text_attr_file(const gchar * filename);
-const gchar *libbalsa_file_get_charset(const gchar * filename);
-
-gchar *libbalsa_lookup_mime_type(const gchar * path);
 
 size_t libbalsa_readfile(FILE * fp, char **buf);
 size_t libbalsa_readfile_nostat(FILE * fp, char **buf);
@@ -140,11 +142,17 @@ gboolean libbalsa_insert_with_url(GtkTextBuffer * buffer,
 				  const char *all_chars,
 				  GtkTextTag * tag,
 				  LibBalsaUrlInsertInfo *url_info);
+#if USE_GREGEX
+void libbalsa_unwrap_selection(GtkTextBuffer * buffer, GRegex * rex);
+gboolean libbalsa_match_regex(const gchar * line, GRegex * rex,
+			      guint * count, guint * index);
+#else                           /* USE_GREGEX */
 void libbalsa_unwrap_selection(GtkTextBuffer * buffer, regex_t * rex);
 gboolean libbalsa_match_regex(const gchar * line, regex_t * rex,
 			      guint * count, guint * index);
+#endif                          /* USE_GREGEX */
 
-int libbalsa_safe_open (const char *path, int flags, mode_t mode);
+int libbalsa_safe_open (const char *path, int flags, mode_t mode, GError **err);
 int libbalsa_lock_file (const char *path, int fd, int excl, int dot, int timeout);
 int libbalsa_unlock_file (const char *path, int fd, int dot);
 int libbalsa_safe_rename (const char *src, const char *target);
@@ -157,29 +165,29 @@ gboolean libbalsa_str_has_prefix(const gchar * str, const gchar * prefix);
 gboolean libbalsa_ia_rfc2821_equal(const InternetAddress * a,
 				   const InternetAddress * b);
 
-/*
- * Face and X-Face header support.
- */
-gchar *libbalsa_get_header_from_path(const gchar * header,
-                                     const gchar * path, gsize * size,
-                                     GError ** err);
-GtkWidget *libbalsa_get_image_from_face_header(const gchar * content,
-                                               GError ** err);
-#if HAVE_COMPFACE
-GtkWidget *libbalsa_get_image_from_x_face_header(const gchar * content,
-                                                 GError ** err);
-#endif                          /* HAVE_COMPFACE */
 
-GQuark libbalsa_image_error_quark(void);
-#define LIBBALSA_IMAGE_ERROR libbalsa_image_error_quark()
-enum LibBalsaImageError {
-    LIBBALSA_IMAGE_ERROR_NO_DATA
-#if HAVE_COMPFACE
-        ,
-    LIBBALSA_IMAGE_ERROR_FORMAT,
-    LIBBALSA_IMAGE_ERROR_BUFFER,
-    LIBBALSA_IMAGE_ERROR_BAD_DATA
-#endif                          /* HAVE_COMPFACE */
-};
+GtkWidget *libbalsa_create_table(guint rows, guint columns);
+GtkWidget *libbalsa_create_label(const gchar * label, GtkWidget * table,
+                                 gint row);
+GtkWidget *libbalsa_create_entry(GtkWidget * table, GCallback func,
+                                 gpointer data, gint row,
+                                 const gchar * initval,
+                                 GtkWidget * hotlabel);
+GtkWidget *libbalsa_create_check(const gchar * label, GtkWidget * table,
+                                 gint row, gboolean initval);
+GtkSizeGroup *libbalsa_create_size_group(GtkWidget * chooser);
+
+void libbalsa_assure_balsa_dir(void);
+gchar *libbalsa_guess_ldap_base(void);
+gchar *libbalsa_guess_ldap_name(void);
+
+gchar *libbalsa_guess_ldif_file(void);
+
+gboolean libbalsa_ldap_exists(const gchar *server);
+
+gboolean libbalsa_path_is_below_dir(const gchar * path, const gchar * dir);
+
+gchar * libbalsa_text_to_html(const gchar * title, const gchar * body, const gchar * lang);
+GString * libbalsa_html_encode_hyperlinks(GString * paragraph);
 
 #endif				/* __LIBBALSA_MISC_H__ */
