@@ -65,6 +65,8 @@ static gboolean accept_low_trust_key(const gchar * name,
 				     GMimeGpgmeContext * ctx);
 static gboolean gpg_updates_trustdb(void);
 static gchar *fix_EMail_info(gchar * str);
+static gboolean have_pub_key_for(gpgme_ctx_t gpgme_ctx,
+				 InternetAddressList * recipients);
 
 
 /* ==== public functions =================================================== */
@@ -141,6 +143,43 @@ body_is_type(LibBalsaMessageBody * body, const gchar * type,
 }
 
 
+/* return TRUE if we can encrypt for every recipient in the recipients list
+ * using protocol */
+gboolean
+libbalsa_can_encrypt_for_all(InternetAddressList * recipients,
+			     gpgme_protocol_t protocol)
+{
+    gpgme_ctx_t gpgme_ctx;
+    gboolean result;
+
+    /* silent paranoia checks */
+    if (!recipients)
+	return TRUE;  /* we can of course encrypt for nobody... */
+#ifndef HAVE_SMIME
+    if (protocol == GPGME_PROTOCOL_OpenPGP)
+	return FALSE;
+#endif
+
+    /* check if gpg is currently available */
+    if (protocol == GPGME_PROTOCOL_OpenPGP && gpg_updates_trustdb())
+	return FALSE;
+
+    /* create the gpgme context and set the protocol */
+    if (gpgme_new(&gpgme_ctx) != GPG_ERR_NO_ERROR)
+	return FALSE;
+    if (gpgme_set_protocol(gpgme_ctx, protocol) != GPG_ERR_NO_ERROR) {
+	gpgme_release(gpgme_ctx);
+	return FALSE;
+    }
+
+    /* loop over all recipients and try to find valid keys */
+    result = have_pub_key_for(gpgme_ctx, recipients);
+    gpgme_release(gpgme_ctx);
+
+    return result;
+}
+
+
 /*
  * Check if body (and eventually its subparts) are RFC 2633 or RFC 3156 signed
  * or encrypted.
@@ -1085,7 +1124,7 @@ libbalsa_signature_info_to_gchar(GMimeGpgmeSigstat * info,
 #include <sys/wait.h>
 #include <fcntl.h>
 
-/* run gpg asynchronously to import a key */
+/* run gpg asynchronously to import or update a key */
 typedef struct _spawned_gpg_T {
     gint child_pid;
     gint standard_error;
@@ -1096,7 +1135,8 @@ typedef struct _spawned_gpg_T {
 static gboolean check_gpg_child(gpointer data);
 
 gboolean
-gpg_run_import_key(const gchar * fingerprint, GtkWindow * parent)
+gpg_keyserver_op(const gchar * fingerprint, gpg_keyserver_action_t action,
+                 GtkWindow * parent)
 {
     gchar **argv;
     spawned_gpg_T *spawned_gpg;
@@ -1106,7 +1146,16 @@ gpg_run_import_key(const gchar * fingerprint, GtkWindow * parent)
     argv = g_new(gchar *, 5);
     argv[0] = g_strdup(GPG_PATH);
     argv[1] = g_strdup("--no-greeting");
-    argv[2] = g_strdup("--recv-keys");
+    switch (action) {
+    case GPG_KEYSERVER_IMPORT:
+        argv[2] = g_strdup("--recv-keys");
+        break;
+    case GPG_KEYSERVER_UPDATE:
+        argv[2] = g_strdup("--refresh-keys");
+        break;
+    default:
+        g_assert_not_reached();
+    }
     argv[3] = g_strdup(fingerprint);
     argv[4] = NULL;
     spawned_gpg = g_new0(spawned_gpg_T, 1);
@@ -1120,7 +1169,7 @@ gpg_run_import_key(const gchar * fingerprint, GtkWindow * parent)
     if (spawnres == FALSE) {
 	libbalsa_information(LIBBALSA_INFORMATION_ERROR,
 			     _
-			     ("Could not launch %s to get the public key %s."),
+			     ("Could not launch %s to query the public key %s."),
 			     GPG_PATH, fingerprint);
 	g_free(spawned_gpg);
 	return FALSE;
@@ -1170,15 +1219,15 @@ check_gpg_child(gpointer data)
 				   GTK_DIALOG_DESTROY_WITH_PARENT,
 				   GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
 				   _
-				   ("Running gpg failed with return value %d:\n%s"),
-				   WEXITSTATUS(status), gpg_message);
+				   ("Running %s failed with return value %d:\n%s"),
+				   GPG_PATH, WEXITSTATUS(status), gpg_message);
     else
 	dialog =
 	    gtk_message_dialog_new(spawned_gpg->parent,
 				   GTK_DIALOG_DESTROY_WITH_PARENT,
 				   GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
-				   _("Running gpg successful:\n%s"),
-				   gpg_message);
+				   _("Running %s successful:\n%s"),
+				   GPG_PATH, gpg_message);
     g_free(gpg_message);
     g_string_free(spawned_gpg->stderr_buf, TRUE);
     g_free(spawned_gpg);
@@ -1692,4 +1741,49 @@ gpg_updates_trustdb(void)
 	return FALSE;
 }
 
+
+/* check if the context contains a public key for the passed recipients */
+#define KEY_IS_OK(k)   (!((k)->expired || (k)->revoked || \
+                          (k)->disabled || (k)->invalid))
+static gboolean
+have_pub_key_for(gpgme_ctx_t gpgme_ctx, InternetAddressList * recipients)
+{
+    gpgme_key_t key;
+    gboolean result = TRUE;
+    time_t now = time(NULL);
+
+    for (; result && recipients; recipients = recipients->next) {
+        InternetAddress *ia = recipients->address;
+
+	/* check all entries in the list, handle groups recursively */
+	if (ia->type == INTERNET_ADDRESS_GROUP)
+	    result = have_pub_key_for(gpgme_ctx, ia->value.members);
+	else if (recipients->address->type == INTERNET_ADDRESS_NAME) {
+	    if (gpgme_op_keylist_start(gpgme_ctx, ia->value.addr, FALSE) !=
+		GPG_ERR_NO_ERROR)
+		return FALSE;
+
+	    result = FALSE;
+	    while (!result &&
+		   gpgme_op_keylist_next(gpgme_ctx, &key) == GPG_ERR_NO_ERROR) {
+		/* check if this key and the relevant subkey are usable */
+		if (KEY_IS_OK(key)) {
+		    gpgme_subkey_t subkey = key->subkeys;
+
+		    while (subkey && !subkey->can_encrypt)
+			subkey = subkey->next;
+
+		    if (subkey && KEY_IS_OK(subkey) && 
+			(subkey->expires == 0 || subkey->expires > now))
+			result = TRUE;
+		}
+		gpgme_key_unref(key);
+	    }
+	    gpgme_op_keylist_end(gpgme_ctx);
+	}
+    }
+
+    return result;
+}
+
 #endif				/* HAVE_GPGME */
