@@ -58,6 +58,8 @@
 gboolean libbalsa_check_crypto_engine(gpgme_protocol_t protocol);
 
 gint libbalsa_message_body_protection(LibBalsaMessageBody * body);
+gboolean libbalsa_can_encrypt_for_all(InternetAddressList * recipients,
+				      gpgme_protocol_t protocol);
 
 /* routines dealing with RFC 2633 and RFC 3156 stuff */
 gboolean libbalsa_sign_mime_object(GMimeObject ** content,
@@ -105,7 +107,15 @@ gchar *libbalsa_signature_info_to_gchar(GMimeGpgmeSigstat * info,
 					const gchar * date_string);
 
 #ifdef HAVE_GPG
-gboolean gpg_run_import_key(const gchar * fingerprint, GtkWindow * parent);
+
+typedef enum {
+    GPG_KEYSERVER_IMPORT = 1,
+    GPG_KEYSERVER_UPDATE
+} gpg_keyserver_action_t;
+
+gboolean gpg_keyserver_op(const gchar * fingerprint,
+                          gpg_keyserver_action_t action,
+                          GtkWindow * parent);
 
 #endif
 #endif				/* HAVE_GPGME */
