@@ -27,15 +27,17 @@
 
 #include <string.h>
 #include <libesmtp.h>
+
 #ifdef HAVE_GNOME
 #include <gnome.h>
 #endif                          /* HAVE_GNOME */
 
-
+#include "libbalsa.h"
+#include "server.h"
 #include "smtp-server.h"
 #include "libbalsa-conf.h"
 #include "misc.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 
 static LibBalsaServerClass *parent_class = NULL;
 
