@@ -44,6 +44,7 @@
 /* Global application structure */
 struct BalsaApplication balsa_app;
 
+#if !HAVE_GTKSPELL
 const gchar *pspell_modules[] = {
     "ispell",
     "aspell"
@@ -54,6 +55,7 @@ const gchar *pspell_suggest_modes[] = {
     "normal",
     "bad-spellers"
 };
+#endif                          /* HAVE_GTKSPELL */
 
 /* ask_password:
    asks the user for the password to the mailbox on given remote server.
@@ -304,9 +306,9 @@ balsa_app_init(void)
 
     balsa_app.toolbar_wrap_button_text = TRUE;
     balsa_app.pwindow_option = WHILERETR;
-    balsa_app.wordwrap = TRUE;
+    balsa_app.wordwrap = FALSE; /* default to format=flowed. */
     balsa_app.wraplength = 72;
-    balsa_app.browse_wrap = TRUE;
+    balsa_app.browse_wrap = FALSE; /* GtkTextView will wrap for us. */
     balsa_app.browse_wrap_length = 79;
     balsa_app.shown_headers = HEADERS_SELECTED;
     balsa_app.show_all_headers = FALSE;
@@ -389,6 +391,7 @@ balsa_app_init(void)
     /* spell check */
 #if HAVE_GTKSPELL
     balsa_app.spell_check_lang = NULL;
+    balsa_app.spell_check_active = FALSE;
 #else                           /* HAVE_GTKSPELL */
     balsa_app.module = SPELL_CHECK_MODULE_ASPELL;
     balsa_app.suggestion_mode = SPELL_CHECK_SUGGEST_NORMAL;
@@ -407,6 +410,9 @@ balsa_app_init(void)
 
     balsa_app.notify_new_mail_sound = 1;
     balsa_app.notify_new_mail_dialog = 0;
+#if GTK_CHECK_VERSION(2, 10, 0)
+    balsa_app.notify_new_mail_icon = 1;
+#endif                          /* GTK_CHECK_VERSION(2, 10, 0) */
 
 #if !GTK_CHECK_VERSION(2, 11, 0)
     /* Tooltips */
@@ -882,3 +888,33 @@ balsa_find_index_by_mailbox(LibBalsaMailbox * mailbox)
     /* didn't find a matching mailbox */
     return NULL;
 }
+
+#if USE_GREGEX
+GRegex *
+balsa_quote_regex_new(void)
+{
+    static GRegex *regex  = NULL;
+    static gchar  *string = NULL;
+
+    if (string && strcmp(string, balsa_app.quote_regex) != 0) {
+        g_free(string);
+        string = NULL;
+        g_regex_unref(regex);
+        regex = NULL;
+    }
+
+    if (!regex) {
+        GError *err = NULL;
+
+        regex = g_regex_new(balsa_app.quote_regex, 0, 0, &err);
+        if (err) {
+            g_warning("quote regex compilation failed: %s", err->message);
+            g_error_free(err);
+            return NULL;
+        }
+        string = g_strdup(balsa_app.quote_regex);
+    }
+
+    return g_regex_ref(regex);
+}
+#endif                          /* USE_GREGEX */
