@@ -208,6 +208,9 @@ extern struct BalsaApplication {
     gint notify_new_mail_sound;
     
     gint notify_new_mail_dialog;
+#if GTK_CHECK_VERSION(2, 10, 0)
+    gint notify_new_mail_icon;
+#endif                          /* GTK_CHECK_VERSION(2, 10, 0) */
 
     /* automatically close mailboxes after XX minutes */
     gboolean close_mailbox_auto;
@@ -298,9 +301,13 @@ extern struct BalsaApplication {
 
     gboolean empty_trash_on_exit;
     gboolean previewpane;
-    gboolean source_escape_specials;
     gboolean debug;
 
+    /* Source viewer */
+    gboolean source_escape_specials;
+    gint source_width;
+    gint source_height;
+
     /* what to do with message window after moving the message */
     MwActionAfterMove mw_action_after_move;
 
@@ -376,7 +383,8 @@ extern struct BalsaApplication {
 
     /* spell checking */
 #if HAVE_GTKSPELL
-    gchar *spell_check_lang;
+    gchar   *spell_check_lang;
+    gboolean spell_check_active;
 #else                           /* HAVE_GTKSPELL */
     SpellCheckModule module;
     SpellCheckSuggestMode suggestion_mode;
@@ -448,4 +456,8 @@ BalsaIndex* balsa_find_index_by_mailbox(LibBalsaMailbox* mailbox);
 
 void  balsa_remove_children_mailbox_nodes(BalsaMailboxNode * mbnode);
 
+#if USE_GREGEX
+GRegex *balsa_quote_regex_new(void);
+#endif                          /* USE_GREGEX */
+
 #endif				/* __BALSA_APP_H__ */
