@@ -57,6 +57,8 @@
 #define BALSA_PIXMAP_MBOX_IN                    "balsa_mbox_in"
 #define BALSA_PIXMAP_MBOX_OUT                   "balsa_mbox_out"
 #define BALSA_PIXMAP_MBOX_SENT                  "balsa_mbox_sent"
+#define BALSA_PIXMAP_MBOX_TRASH                 "balsa_mbox_trash"
+#define BALSA_PIXMAP_MBOX_TRASH_FULL            "balsa_mbox_trash_full"
 #define BALSA_PIXMAP_MBOX_TRAY_EMPTY            "balsa_mbox_tray_empty"
 #define BALSA_PIXMAP_MBOX_TRAY_FULL             "balsa_mbox_tray_full"
 #define BALSA_PIXMAP_MBOX_DIR_OPEN              "balsa_mbox_dir_open"
@@ -84,8 +86,13 @@
 #define BALSA_PIXMAP_BOOK_BLUE                  "balsa_book_blue"
 #define BALSA_PIXMAP_BOOK_OPEN                  "balsa_book_open"
 
+#define BALSA_PIXMAP_DROP_DOWN                  "balsa_drop_down"
+
 void register_balsa_pixmaps(void);
 void register_balsa_pixbufs(GtkWidget * widget);
 void balsa_icon_create(const gchar ** data, GdkPixmap ** pmap,
                        GdkBitmap ** bmap);
+#if GTK_CHECK_VERSION(2, 8, 0)
+const gchar * balsa_icon_id(const gchar * name);
+#endif                          /* GTK_CHECK_VERSION(2, 8, 0) */
 #endif
