@@ -387,8 +387,9 @@ bndx_instance_init(BalsaIndex * index)
 
     /* handle select row signals to display message in the window
      * preview pane */
-    g_signal_connect(selection, "changed",
-                     G_CALLBACK(bndx_selection_changed), index);
+    index->selection_changed_id =
+        g_signal_connect(selection, "changed",
+                         G_CALLBACK(bndx_selection_changed), index);
 
     /* we want to handle button presses to pop up context menus if
      * necessary */
@@ -624,14 +625,16 @@ bndx_scroll_idle(BalsaIndex * index)
     GtkTreePath *path;
 
     gdk_threads_enter();
-
-    row_ref = g_object_get_data(G_OBJECT(index), BALSA_INDEX_ROW_REF_KEY);
-    if (row_ref && (path = gtk_tree_row_reference_get_path(row_ref))) {
-        gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(index), path, NULL,
-                                     FALSE, 0, 0);
-        gtk_tree_path_free(path);
+    /* The index might have been destroyed since the idle function was
+       registered. Check whether we still exist... */
+    if(gtk_tree_view_get_model(GTK_TREE_VIEW(index))) {
+        row_ref = g_object_get_data(G_OBJECT(index), BALSA_INDEX_ROW_REF_KEY);
+        if (row_ref && (path = gtk_tree_row_reference_get_path(row_ref))) {
+            gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(index), path, NULL,
+                                         FALSE, 0, 0);
+            gtk_tree_path_free(path);
+        }
     }
-
     g_object_set_data(G_OBJECT(index), BALSA_INDEX_ROW_REF_KEY, NULL);
     g_object_unref(index);
 
@@ -772,7 +775,8 @@ balsa_index_new(void)
  * the index, or the last message if none is unread, and selects
  * it. Since this routine is usually called from a thread, we have to
  * take care and and make sure the row is selected from the main
- * thread only.
+ * thread only. And we also check whether the mailbox hasn't been
+ * destroyed by now, of course.
  */
 struct view_on_open_data {
     BalsaIndex *bindex;
@@ -782,8 +786,10 @@ static gboolean
 bi_view_on_open(struct view_on_open_data *data)
 {
     gdk_threads_enter();
-    bndx_select_row(data->bindex, data->path);
-    gtk_tree_path_free(data->path);
+    if(gtk_tree_view_get_model(GTK_TREE_VIEW(data->bindex))) {
+        bndx_select_row(data->bindex, data->path);
+        gtk_tree_path_free(data->path);
+    }
     g_object_unref(data->bindex);
     g_free(data);
     gdk_threads_leave();
@@ -1439,7 +1445,9 @@ bndx_view_source(gpointer data)
         if (!message)
             continue;
 	libbalsa_show_message_source(message, balsa_app.message_font,
-				     &balsa_app.source_escape_specials);
+				     &balsa_app.source_escape_specials,
+                                     &balsa_app.source_width,
+                                     &balsa_app.source_height);
         g_object_unref(message);
     }
     balsa_index_selected_msgnos_free(index, selected);
@@ -1849,6 +1857,41 @@ bndx_popup_menu_create(BalsaIndex * index)
  * set sensitivity of menuitems on the popup
  * menu, and populate the mru submenu
  */
+
+/* If the menu is popped up in response to a keystroke, center it
+ * below the headers of the tree-view.
+ */
+static void
+bndx_popup_position_func(GtkMenu * menu, gint * x, gint * y,
+                         gboolean * push_in, gpointer user_data)
+{
+    GtkWidget *bindex = GTK_WIDGET(user_data);
+    GdkScreen *screen = gtk_widget_get_screen(bindex);
+    GtkRequisition req;
+    gint monitor_num;
+    GdkRectangle monitor;
+
+    g_return_if_fail(GTK_WIDGET_REALIZED(bindex));
+
+    gdk_window_get_origin(gtk_tree_view_get_bin_window
+                          (GTK_TREE_VIEW(bindex)), x, y);
+
+    gtk_widget_size_request(GTK_WIDGET(menu), &req);
+
+    *x += (bindex->allocation.width - req.width) / 2;
+
+    monitor_num = gdk_screen_get_monitor_at_point(screen, *x, *y);
+    gtk_menu_set_monitor(menu, monitor_num);
+    gdk_screen_get_monitor_geometry(screen, monitor_num, &monitor);
+
+    *x = CLAMP(*x, monitor.x,
+               monitor.x + MAX(0, monitor.width - req.width));
+    *y = CLAMP(*y, monitor.y,
+               monitor.y + MAX(0, monitor.height - req.height));
+
+    *push_in = FALSE;
+}
+
 static void
 bndx_do_popup(BalsaIndex * index, GdkEventButton * event)
 {
@@ -1859,8 +1902,6 @@ bndx_do_popup(BalsaIndex * index, GdkEventButton * event)
     gboolean any;
     gboolean any_deleted = FALSE;
     gboolean any_not_deleted = FALSE;
-    gint event_button;
-    guint event_time;
     GArray *selected = balsa_index_selected_msgnos_new(index);
     guint i;
 
@@ -1907,15 +1948,13 @@ bndx_do_popup(BalsaIndex * index, GdkEventButton * event)
 
     gtk_widget_show_all(menu);
 
-    if (event) {
-        event_button = event->button;
-        event_time = event->time;
-    } else {
-        event_button = 0;
-        event_time = gtk_get_current_event_time();
-    }
-    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
-                   event_button, event_time);
+    if (event)
+        gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
+                       event->button, event->time);
+    else
+        gtk_menu_popup(GTK_MENU(menu), NULL, NULL,
+                       bndx_popup_position_func, index,
+                       0, gtk_get_current_event_time());
 }
 
 static GtkWidget *
@@ -2213,7 +2252,7 @@ bndx_pipe(LibBalsaMailbox * mailbox, guint msgno, const gchar * pipe_cmd)
         fprintf(stderr, "popen failed for message %d\n", msgno);
     } else {
         GMimeStream *stream =
-            libbalsa_mailbox_get_message_stream(mailbox, msgno);
+            libbalsa_mailbox_get_message_stream(mailbox, msgno, TRUE);
         GMimeStream *pipe;
 
         g_return_if_fail(stream);
@@ -2370,6 +2409,9 @@ balsa_index_ensure_visible(BalsaIndex * index)
     GdkRectangle rect;
     GtkTreePath *path = NULL;
 
+    if (!GTK_WIDGET_REALIZED(tree_view))
+        return;
+
     if (!bndx_find_current_msgno(index, &path, NULL)) {
         /* Current message not displayed, make sure that something
            else is... */
@@ -2410,3 +2452,26 @@ balsa_index_ensure_visible(BalsaIndex * index)
         gtk_tree_path_free(path);
     }
 }
+
+void
+balsa_index_select_all(BalsaIndex * bindex)
+{
+    GtkTreeView *tree_view = GTK_TREE_VIEW(bindex);
+    GtkTreeSelection *selection = gtk_tree_view_get_selection(tree_view);
+
+    gtk_tree_view_expand_all(tree_view);
+    g_signal_handler_block(selection, bindex->selection_changed_id);
+    gtk_tree_selection_select_all(selection);
+    g_signal_handler_unblock(selection, bindex->selection_changed_id);
+    g_signal_emit(bindex, balsa_index_signals[INDEX_CHANGED], 0);
+}
+
+gint
+balsa_index_count_selected_messages(BalsaIndex * bindex)
+{
+    g_return_val_if_fail(BALSA_IS_INDEX(bindex), -1);
+
+    return
+        gtk_tree_selection_count_selected_rows(gtk_tree_view_get_selection
+                                               (GTK_TREE_VIEW(bindex)));
+}
