@@ -36,7 +36,11 @@
 #include <gtksourceview/gtksourceview.h>
 #include <gtksourceview/gtksourcebuffer.h>
 #include <gtksourceview/gtksourcelanguage.h>
-#include <gtksourceview/gtksourcelanguagesmanager.h>
+#if (HAVE_GTKSOURCEVIEW == 1)
+#  include <gtksourceview/gtksourcelanguagesmanager.h>
+#else
+#  include <gtksourceview/gtksourcelanguagemanager.h>
+#endif
 #endif
 
 
@@ -123,7 +127,11 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
     size_t alloced;
     BalsaMimeWidget *mw;
     GtkTextBuffer *buffer;
+#if USE_GREGEX
+    GRegex *rex;
+#else                           /* USE_GREGEX */
     regex_t rex;
+#endif                          /* USE_GREGEX */
     GList *url_list = NULL;
     const gchar *target_cs;
     GError *err = NULL;
@@ -156,6 +164,7 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
                                        GTK_POLICY_ALWAYS);
         return bm_widget_new_html(bm, mime_body, ptr, alloced);
 #else
+	g_free(ptr);
         return NULL;
 #endif
     }
@@ -227,11 +236,17 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
     gtk_text_buffer_create_tag(buffer, "soft", NULL, NULL);
     allocate_quote_colors(GTK_WIDGET(bm), balsa_app.quoted_color,
 			  0, MAX_QUOTED_COLOR - 1);
+#if USE_GREGEX
+    if (!(rex = balsa_quote_regex_new()))
+	gtk_text_buffer_insert_at_cursor(buffer, ptr, -1);
+#else                           /* USE_GREGEX */
     if (regcomp(&rex, balsa_app.quote_regex, REG_EXTENDED) != 0) {
 	g_warning
 	    ("part_info_init_mimetext: quote regex compilation failed.");
 	gtk_text_buffer_insert_at_cursor(buffer, ptr, -1);
-    } else {
+    }
+#endif                          /* USE_GREGEX */
+    else {
 	gchar *line_start;
 	LibBalsaUrlInsertInfo url_info;
 	GList * cite_bars_list;
@@ -277,7 +292,11 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 		guint cite_idx;
 
 		/* get the cite level only for text/plain parts */
+#if USE_GREGEX
+		libbalsa_match_regex(this_line, rex, &quote_level, &cite_idx);
+#else                           /* USE_GREGEX */
 		libbalsa_match_regex(this_line, &rex, &quote_level, &cite_idx);
+#endif                          /* USE_GREGEX */
 	    
 		/* check if the citation level changed */
 		if (cite_level != quote_level) {
@@ -339,7 +358,11 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 	    g_signal_connect_after(G_OBJECT(mw->widget), "expose-event",
 				   G_CALLBACK(draw_cite_bars), cite_bars_list);
 	}
+#if USE_GREGEX
+	g_regex_unref(rex);
+#else                           /* USE_GREGEX */
 	regfree(&rex);
+#endif                          /* USE_GREGEX */
     }
 
     prepare_url_offsets(buffer, url_list);
@@ -380,7 +403,7 @@ balsa_mime_widget_new_text(BalsaMessage * bm, LibBalsaMessageBody * mime_body,
 static GtkWidget *
 create_text_widget(const char * content_type)
 {
-#if HAVE_GTKSOURCEVIEW
+#if (HAVE_GTKSOURCEVIEW == 1)
     static GtkSourceLanguagesManager * lm = NULL;
     GtkSourceLanguage * lang;
     GtkSourceBuffer * buffer;
@@ -405,7 +428,53 @@ create_text_widget(const char * content_type)
     widget = gtk_source_view_new_with_buffer(buffer);
     g_object_unref(buffer);
     return widget;
-#else
+#elif (HAVE_GTKSOURCEVIEW == 2)
+    static GtkSourceLanguageManager * lm = NULL;
+    static const gchar * const * lm_ids = NULL;
+    GtkWidget * widget = NULL;
+    gint n;
+
+    /* we use or own highlighting for text/plain */
+    if (!g_ascii_strcasecmp(content_type, "text/plain"))
+	return gtk_text_view_new();
+    
+    /* try to initialise the source language manager and return a "simple"
+     * text view if this fails */
+    if (!lm)
+	lm = gtk_source_language_manager_get_default();
+    if (lm && !lm_ids)
+	lm_ids = gtk_source_language_manager_get_language_ids(lm);
+    if (!lm_ids)
+	return gtk_text_view_new();
+    
+    /* search for a language supporting our mime type */
+    for (n = 0; !widget && lm_ids[n]; n++) {
+	GtkSourceLanguage * src_lang =
+	    gtk_source_language_manager_get_language(lm, lm_ids[n]);
+	gchar ** mime_types;
+
+	if (src_lang &&
+	    (mime_types = gtk_source_language_get_mime_types(src_lang))) {
+	    gint k;
+
+	    for (k = 0;
+		 mime_types[k] && g_ascii_strcasecmp(mime_types[k], content_type);
+		 k++);
+	    if (mime_types[k]) {
+		GtkSourceBuffer * buffer =
+		    gtk_source_buffer_new_with_language(src_lang);
+		
+		gtk_source_buffer_set_highlight_syntax(buffer, TRUE);
+		widget = gtk_source_view_new_with_buffer(buffer);
+		g_object_unref(buffer);
+	    }
+	    g_strfreev(mime_types);
+	}
+    }
+    
+    /* fall back to the simple text view if the mime type is not supported */
+    return widget ? widget : gtk_text_view_new();
+#else /* no GtkSourceview */
     return gtk_text_view_new();
 #endif
 }
