@@ -32,7 +32,7 @@
 #include "balsa-app.h"
 #include "filter-export.h"
 
-#include "i18n.h"	/* Must come after balsa-app.h. */
+#include <glib/gi18n.h>	/* Must come after balsa-app.h. */
 
 /* To prevent user from silmultaneously edit/export filters */
 extern gboolean fex_already_open;
