@@ -53,6 +53,8 @@
 
 #include "libbalsa.h"
 #include "imap-server.h"
+#include "mailbox-filter.h"
+#include "libbalsa-conf.h"
 #include <glib/gi18n.h>
 
 struct _BalsaMailboxConfView {
@@ -371,6 +373,7 @@ mailbox_conf_delete(BalsaMailboxNode * mbnode)
     gint button;
     GtkWidget *ask;
     LibBalsaMailbox* mailbox = mbnode->mailbox;
+    gchar *url, *group;
 
     if(BALSA_IS_MAILBOX_SPECIAL(mailbox)) {
 	balsa_information(
@@ -449,6 +452,9 @@ mailbox_conf_delete(BalsaMailboxNode * mbnode)
     if ( button < 0)
 	return;
 
+    /* Save the mailbox URL */
+    url = g_strdup(mailbox->url ? mailbox->url : mailbox->name);
+
     /* Delete it from the config file and internal nodes */
     config_mailbox_delete(mailbox);
 
@@ -492,6 +498,18 @@ mailbox_conf_delete(BalsaMailboxNode * mbnode)
     } else
 	balsa_mblist_mailbox_node_remove(mbnode);
     update_mail_servers();
+
+    /* Clean up filters */
+    group = mailbox_filters_section_lookup(url);
+    if (group) {
+        libbalsa_conf_remove_group(group);
+        g_free(group);
+    }
+
+    /* Remove view */
+    config_view_remove(url);
+
+    g_free(url);
 }
 
 #define MCW_RESPONSE 1
@@ -975,9 +993,9 @@ mailbox_conf_add(MailboxConfWindow * mcw)
 	    return;
 	}
 
-	save_to_config = balsa_app.local_mail_directory == NULL
-	    || strncmp(balsa_app.local_mail_directory, path,
-		       strlen(balsa_app.local_mail_directory)) != 0;
+	save_to_config =
+            !libbalsa_path_is_below_dir(path,
+                                        balsa_app.local_mail_directory);
         printf("Save to config: %d\n", save_to_config);
 	mcw->mailbox->name = g_path_get_basename(path);
 	balsa_mailbox_local_append(mcw->mailbox);
