@@ -32,7 +32,7 @@
 #include "save-restore.h"
 #include "filter.h"
 #include "imap-server.h"
-#include "i18n.h"
+#include <glib/gi18n.h>
 #include "libbalsa-conf.h"
 
 /* MailboxNode object is a GUI representation of a mailbox, or entire 
@@ -341,7 +341,7 @@ check_local_path(const gchar * path, guint depth)
         return TRUE;
 
     len = strlen(balsa_app.local_mail_directory);
-    if (!strncmp(path, balsa_app.local_mail_directory, len)
+    if (libbalsa_path_is_below_dir(path, balsa_app.local_mail_directory)
 	&& !strchr(&path[++len], G_DIR_SEPARATOR))
 	/* Top level folder. */
 	return TRUE;
@@ -361,10 +361,13 @@ check_local_path(const gchar * path, guint depth)
     return cpi.must_scan;
 }
 
-static void
+static gboolean
 mark_local_path(BalsaMailboxNode *mbnode)
 {
+    if (mbnode->scanned)
+        return FALSE;
     mbnode->scanned = TRUE;
+    return TRUE;
 }
 
 /** Read local directory in search for mailboxes. Never does any
@@ -469,6 +472,14 @@ imap_dir_cb(BalsaMailboxNode* mb)
     GSList *list;
     GError *error = NULL;
     imap_scan_tree imap_tree = { NULL, '.' };
+    GtkStatusbar *statusbar;
+    guint context_id;
+
+    if (!balsa_app.main_window)
+        return;
+
+    statusbar = GTK_STATUSBAR(balsa_app.main_window->statusbar);
+    context_id = gtk_statusbar_get_context_id(statusbar, "MailboxNode imap");
 
     if(restore_children_from_cache(mb))
         return;
@@ -476,7 +487,7 @@ imap_dir_cb(BalsaMailboxNode* mb)
     while(mroot->parent)
 	mroot = mroot->parent;
     msg = g_strdup_printf(_("Scanning %s. Please wait..."), mroot->name);
-    gnome_appbar_push(balsa_app.appbar, msg);
+    gtk_statusbar_push(statusbar, context_id, msg);
     g_free(msg);
 
     g_return_if_fail(mb->server);
@@ -498,7 +509,7 @@ imap_dir_cb(BalsaMailboxNode* mb)
                              error->message);
         g_error_free(error);
         imap_scan_destroy_tree(&imap_tree);
-        gnome_appbar_pop(balsa_app.appbar);
+        gtk_statusbar_pop(statusbar, context_id);
         return;
     }
 
@@ -528,7 +539,7 @@ imap_dir_cb(BalsaMailboxNode* mb)
         printf("imap_dir_cb:  main mailbox node %s mailbox is %p\n", 
                mb->name, mb->mailbox);
     if(balsa_app.debug) printf("%d: Scanning done.\n", (int)time(NULL));
-    gnome_appbar_pop(balsa_app.appbar);
+    gtk_statusbar_pop(statusbar, context_id);
 
     /* We can save the cache now... */
     config_folder_update(mroot);
@@ -784,7 +795,6 @@ restore_children_from_cache(BalsaMailboxNode *mn)
         }
     }
     
-    printf("%s's children restored from cache.\n", mn->name);
     return TRUE;
 }
 
@@ -1205,7 +1215,9 @@ add_local_mailbox(BalsaMailboxNode *root, const gchar * name,
 	    g_print(_("Local mailbox %s loaded as: %s\n"),
 		    mailbox->name,
 		    g_type_name(G_OBJECT_TYPE(mailbox)));
-	if (balsa_app.check_mail_upon_startup) {
+        if (balsa_app.check_mail_upon_startup
+            && libbalsa_mailbox_get_subscribe(mailbox) !=
+            LB_MAILBOX_SUBSCRIBE_NO) {
             g_object_ref(mailbox);
             g_idle_add((GSourceFunc) mailbox_check_idle, mailbox);
         }
