@@ -45,8 +45,6 @@
 
 #include <glib/gi18n.h>
 
-#include "address-entry.h"
-
 #include "balsa-app.h"
 #include "balsa-icons.h"
 #include "balsa-index.h"
@@ -137,7 +135,7 @@ accel_map_save(void)
 static void
 balsa_handle_automation_options() {
    CORBA_Object factory;
-   CORBA_Environment ev;
+   CORBA_Environment ev, *ev_p = &ev;
    BonoboObject *balsacomposer;
    BonoboObject *balsaapp;
  
@@ -148,7 +146,7 @@ balsa_handle_automation_options() {
 	Bonobo_ACTIVATION_FLAG_EXISTING_ONLY,
 	NULL, &ev);
    
-   if ( !(BONOBO_EX (&ev) || factory == CORBA_OBJECT_NIL) ) {
+   if ( !(BONOBO_EX (ev_p) || factory == CORBA_OBJECT_NIL) ) {
        /* there already is a server. good */
        CORBA_Object app;
        printf("Another Balsa found. Talking to it...\n");
@@ -286,7 +284,7 @@ balsa_init(int argc, char **argv)
 	{"compose", 'm', 0, G_OPTION_ARG_STRING, &(opt_compose_email),
 	 N_("Compose a new email to EMAIL@ADDRESS"), "EMAIL@ADDRESS"},
 	{"attach", 'a', 0, G_OPTION_ARG_FILENAME_ARRAY, &(attach_vect),
-	 N_("Attach file at PATH"), "PATH"},
+	 N_("Attach file at URI"), "URI"},
 	{"open-mailbox", 'o', 0, G_OPTION_ARG_STRING,
          &(cmd_line_open_mailboxes),
 	 N_("Opens MAILBOXNAME"), N_("MAILBOXNAME")},
@@ -628,6 +626,7 @@ periodic_expunge_cb(void)
  * Initialize the progress bar and set text.
  */
 static GTimeVal prev_time_val;
+static gdouble  min_fraction;
 static void
 balsa_progress_set_text(LibBalsaProgress * progress, const gchar * text,
                         guint total)
@@ -644,6 +643,7 @@ balsa_progress_set_text(LibBalsaProgress * progress, const gchar * text,
     if (!text || total >= LIBBALSA_PROGRESS_MIN_COUNT)
         rc = balsa_window_setup_progress(balsa_app.main_window, text);
     g_get_current_time(&prev_time_val);
+    min_fraction = LIBBALSA_PROGRESS_MIN_UPDATE_STEP;
     gdk_threads_leave();
 
     *progress = (text && rc) ?
@@ -658,17 +658,23 @@ static void
 balsa_progress_set_fraction(LibBalsaProgress * progress, gdouble fraction)
 {
     GTimeVal time_val;
-    gdouble elapsed;
+    guint elapsed;
+
+    if (*progress == LIBBALSA_PROGRESS_NO)
+        return;
+
+    if (fraction > 0.0 && fraction < min_fraction)
+        return;
 
     g_get_current_time(&time_val);
     elapsed = time_val.tv_sec - prev_time_val.tv_sec;
-    elapsed += 1e-6 * (time_val.tv_usec - prev_time_val.tv_usec);
-
-    if (*progress == LIBBALSA_PROGRESS_NO
-        || (fraction != 0 && fraction != 1
-            && elapsed < LIBBALSA_PROGRESS_MIN_UPDATE))
+    elapsed *= G_USEC_PER_SEC;
+    elapsed += time_val.tv_usec - prev_time_val.tv_usec;
+    if (elapsed < LIBBALSA_PROGRESS_MIN_UPDATE_USECS)
         return;
-    prev_time_val = time_val;
+
+    g_time_val_add(&time_val, LIBBALSA_PROGRESS_MIN_UPDATE_USECS);
+    min_fraction += LIBBALSA_PROGRESS_MIN_UPDATE_STEP;
 
     gdk_threads_enter();
     if (balsa_app.main_window)
@@ -677,6 +683,19 @@ balsa_progress_set_fraction(LibBalsaProgress * progress, gdouble fraction)
     gdk_threads_leave();
 }
 
+static void
+balsa_progress_set_activity(gboolean set, const gchar * text)
+{
+    gdk_threads_enter();
+    if (balsa_app.main_window) {
+        if (set)
+            balsa_window_increase_activity(balsa_app.main_window, text);
+        else
+            balsa_window_decrease_activity(balsa_app.main_window, text);
+    }
+    gdk_threads_leave();
+}
+
 #if defined(ENABLE_NLS) && defined(HAVE_GPGME)
 static const gchar *
 get_utf8_locale(int category)
@@ -769,11 +788,10 @@ main(int argc, char *argv[])
 
     libbalsa_progress_set_text     = balsa_progress_set_text;
     libbalsa_progress_set_fraction = balsa_progress_set_fraction;
+    libbalsa_progress_set_activity = balsa_progress_set_activity;
     
     /* checking for valid config files */
     config_init(cmd_get_stats);
-    libbalsa_address_entry_set_address_book_list(balsa_app.
-                                                 address_book_list);
 
     libbalsa_mailbox_view_table =
 	g_hash_table_new_full(g_str_hash, g_str_equal,
@@ -841,7 +859,7 @@ main(int argc, char *argv[])
         for(lst = opt_attach_list; lst; lst = g_slist_next(lst))
             add_attachment(snd, lst->data, FALSE, NULL);
 	snd->quit_on_close = FALSE;
-    };
+    }
     gtk_widget_show(window);
 
     if (cmd_check_mail_on_startup || balsa_app.check_mail_upon_startup)
