@@ -58,6 +58,7 @@ static void view_msg_source_cb         (GtkAction * action, gpointer data);
 
 static void copy_cb                    (GtkAction * action, MessageWindow * mw);
 static void select_all_cb              (GtkAction * action, gpointer);
+static void mw_find_in_message_cb      (GtkAction * action, gpointer data);
 
 static void mw_header_activate_cb      (GtkAction * action, gpointer data);
 
@@ -251,6 +252,9 @@ static const GtkActionEntry entries[] = {
      G_CALLBACK(copy_cb)},
     {"SelectAll", NULL, N_("Select _All"), "<control>A", NULL,
      G_CALLBACK(select_all_cb)},
+    {"FindInMessage", NULL, N_("_Find in message"), "slash",
+     N_("Find a string in this message"),
+     G_CALLBACK(mw_find_in_message_cb)},
 #ifdef HAVE_GTKHTML
     {"ZoomIn", GTK_STOCK_ZOOM_IN, N_("Zoom _In"), "<control>plus",
      N_("Increase magnification"), G_CALLBACK(mw_zoom_in_cb)},
@@ -339,6 +343,8 @@ static const char *ui_description =
 "    <menu action='EditMenu'>"
 "      <menuitem action='Copy'/>"
 "      <menuitem action='SelectAll'/>"
+"      <separator/>"
+"      <menuitem action='FindInMessage'/>"
 "    </menu>"
 "    <menu action='ViewMenu'>"
 "      <menuitem action='Wrap'/>"
@@ -785,7 +791,9 @@ view_msg_source_cb(GtkAction * action, gpointer data)
 {
     MessageWindow *mw = (MessageWindow *) data;
     libbalsa_show_message_source(mw->message, balsa_app.message_font,
-                                 &balsa_app.source_escape_specials);
+                                 &balsa_app.source_escape_specials,
+                                 &balsa_app.source_width,
+                                 &balsa_app.source_height);
 }
 
 static void
@@ -812,7 +820,7 @@ mw_header_activate_cb(GtkAction * action, gpointer data)
             gtk_radio_action_get_current_value(GTK_RADIO_ACTION(action));
         MessageWindow *mw = (MessageWindow *) data;
 
-        balsa_app.shown_headers = sh;
+        mw->headers_shown = sh;
         reset_show_all_headers(mw);
         balsa_message_set_displayed_headers(BALSA_MESSAGE(mw->bmessage),
                                             sh);
@@ -880,6 +888,14 @@ select_all_cb(GtkAction * action, gpointer data)
     balsa_window_select_all(GTK_WINDOW(mw->window));
 }
 
+static void
+mw_find_in_message_cb(GtkAction * action, gpointer data)
+{
+    MessageWindow *mw = (MessageWindow *) (data);
+
+    balsa_message_find_in_message(BALSA_MESSAGE(mw->bmessage));
+}
+
 static void
 mw_set_selected(MessageWindow * mw, void (*select_func) (BalsaIndex *))
 {
@@ -970,16 +986,10 @@ static void
 show_all_headers_tool_cb(GtkToggleAction * action, gpointer data)
 {
     MessageWindow *mw = (MessageWindow *) (data);
-
-    if (gtk_toggle_action_get_active(action)) {
-        mw->show_all_headers = TRUE;
-	balsa_message_set_displayed_headers(BALSA_MESSAGE(mw->bmessage),
-					    HEADERS_ALL);
-    } else {
-        mw->show_all_headers = FALSE;
-	balsa_message_set_displayed_headers(BALSA_MESSAGE(mw->bmessage),
-					    mw->headers_shown);
-    }
+    mw->show_all_headers = gtk_toggle_action_get_active(action);
+    balsa_message_set_displayed_headers(BALSA_MESSAGE(mw->bmessage),
+                                        mw->show_all_headers ?
+                                        HEADERS_ALL : mw->headers_shown);
 }
 
 static void
