@@ -52,16 +52,20 @@
 
 #include <libbalsa.h>
 #include "html.h"
-#ifdef HAVE_PCRE
-#  include <pcreposix.h>
-#else
-#  include <sys/types.h>
-#  include <regex.h>
-#endif
+
+#if !USE_GREGEX
+#  ifdef HAVE_PCRE
+#    include <pcreposix.h>
+#  else
+#    include <sys/types.h>
+#    include <regex.h>
+#  endif
+#endif                          /* USE_GREGEX */
+
 #include "quote-color.h"
 
 #include <string.h>
-#include "i18n.h"
+#include <glib/gi18n.h>
 
 #define BALSA_PRINT_TYPE_HEADER     1
 #define BALSA_PRINT_TYPE_SEPARATOR  2
@@ -783,16 +787,25 @@ print_wrap_body(gchar * str, GnomeFont * font, gint width, gint tab_width)
 {
     gchar *ptr, *line = str;
     gchar *eol;
+#if USE_GREGEX
+    GRegex *rex = NULL;
+#else                           /* USE_GREGEX */
     regex_t rex;
+#endif                          /* USE_GREGEX */
     gboolean checkQuote = balsa_app.print_highlight_cited;
     GList *wrappedLines = NULL;
     gdouble space_width = gnome_font_get_width_utf8(font, " ");
  
+#if USE_GREGEX
+    if (checkQuote && !(rex = balsa_quote_regex_new()))
+        checkQuote = FALSE;
+#else                           /* USE_GREGEX */
     if (checkQuote)
 	if (regcomp(&rex, balsa_app.quote_regex, REG_EXTENDED) != 0) {
 	    g_warning("quote regex compilation failed.");
 	    checkQuote = FALSE;
 	}
+#endif                          /* USE_GREGEX */
     
     g_strchomp(str);
     while (line) {
@@ -804,7 +817,11 @@ print_wrap_body(gchar * str, GnomeFont * font, gint width, gint tab_width)
 	if (eol)
 	    *eol = '\0';
 	ptr = line;
+#if USE_GREGEX
+	lineInfo->quoteLevel = checkQuote ? is_a_quote(ptr, rex) : 0;
+#else                           /* USE_GREGEX */
 	lineInfo->quoteLevel = checkQuote ? is_a_quote(ptr, &rex) : 0;
+#endif                          /* USE_GREGEX */
 	while (*ptr) {
 	    gint pos = 0;
 	    gint last_space = 0;
@@ -853,7 +870,11 @@ print_wrap_body(gchar * str, GnomeFont * font, gint width, gint tab_width)
     }
 
     if (checkQuote)
+#if USE_GREGEX
+        g_regex_unref(rex);
+#else                           /* USE_GREGEX */
 	regfree(&rex);
+#endif                          /* USE_GREGEX */
 
     return g_list_reverse(wrappedLines);
 }
