@@ -30,15 +30,22 @@ extern "C" {
 #include <unistd.h>
 #endif
 #include "config.h"
-#ifdef HAVE_PCRE
-#  include <pcreposix.h>
-#else
-#  include <sys/types.h>
-#  include <regex.h>
-#endif
+
+#if !USE_GREGEX
+#  ifdef HAVE_PCRE
+#    include <pcreposix.h>
+#  else
+#    include <sys/types.h>
+#    include <regex.h>
+#  endif
+#endif                          /* USE_GREGEX */
 
     extern void make_gradient(GdkColor colors[], gint, gint);
-    extern gint is_a_quote(const gchar *, regex_t * rex);
+#if USE_GREGEX
+    extern guint is_a_quote(const gchar *, GRegex * rex);
+#else                           /* USE_GREGEX */
+    extern guint is_a_quote(const gchar *, regex_t * rex);
+#endif                          /* USE_GREGEX */
     extern void allocate_quote_colors(GtkWidget *, GdkColor color[], gint, gint);
 
 
