@@ -195,11 +195,10 @@ config_mailbox_set_as_special(LibBalsaMailbox * mailbox, specialType which)
     if (*special) {
 	g_free((*special)->config_prefix); 
 	(*special)->config_prefix = NULL;
-        if(balsa_app.local_mail_directory == NULL
-           || !LIBBALSA_IS_MAILBOX_LOCAL(*special)
-           || strncmp(balsa_app.local_mail_directory, 
-                      libbalsa_mailbox_local_get_path(*special),
-                      strlen(balsa_app.local_mail_directory)) != 0) 
+        if (!LIBBALSA_IS_MAILBOX_LOCAL(*special)
+            || !libbalsa_path_is_below_dir(libbalsa_mailbox_local_get_path
+                                           (*special),
+                                           balsa_app.local_mail_directory))
             config_mailbox_add(*special, NULL);
 	g_object_remove_weak_pointer(G_OBJECT(*special), (gpointer) special);
 
@@ -805,6 +804,8 @@ config_global_load(void)
     libbalsa_conf_push_group("SourcePreview");
     balsa_app.source_escape_specials = 
         libbalsa_conf_get_bool("EscapeSpecials=true");
+    balsa_app.source_width  = libbalsa_conf_get_int("Width=500");
+    balsa_app.source_height = libbalsa_conf_get_int("Height=400");
     libbalsa_conf_pop_group();
 
     /* Printing options ... */
@@ -875,6 +876,10 @@ config_global_load(void)
 #if HAVE_GTKSPELL
     balsa_app.spell_check_lang =
         libbalsa_conf_get_string("SpellCheckLanguage");
+    balsa_app.spell_check_active =
+        libbalsa_conf_get_bool_with_default("SpellCheckActive", &def_used);
+    if (def_used)
+        balsa_app.spell_check_active = balsa_app.spell_check_lang != NULL;
 #else                           /* HAVE_GTKSPELL */
     balsa_app.module = d_get_gint("PspellModule", DEFAULT_PSPELL_MODULE);
     balsa_app.suggestion_mode =
@@ -905,6 +910,10 @@ config_global_load(void)
 	d_get_gint("NewMailNotificationDialog", 0);
     balsa_app.notify_new_mail_sound =
 	d_get_gint("NewMailNotificationSound", 1);
+#if GTK_CHECK_VERSION(2, 10, 0)
+    balsa_app.notify_new_mail_icon =
+	d_get_gint("NewMailNotificationIcon", 1);
+#endif                          /* GTK_CHECK_VERSION(2, 10, 0) */
     balsa_app.check_mail_upon_startup =
 	libbalsa_conf_get_bool("OnStartup=false");
     balsa_app.check_mail_auto = libbalsa_conf_get_bool("Auto=false");
@@ -1015,7 +1024,7 @@ config_global_load(void)
     balsa_app.quote_str = libbalsa_conf_get_string("QuoteString=> ");
     g_free(balsa_app.compose_headers);
     balsa_app.compose_headers =
-	libbalsa_conf_get_string("ComposeHeaders=to subject cc");
+	libbalsa_conf_get_string("ComposeHeaders=Recipients Subject");
 
     /* Obsolete. */
     libbalsa_conf_get_bool_with_default("RequestDispositionNotification=false",
@@ -1107,6 +1116,18 @@ config_global_load(void)
                          "you will need to set them again."));
             libbalsa_conf_set_bool("GtkUIManager", TRUE);
         }
+        if (!libbalsa_conf_get_bool("LibBalsaAddressView")) {
+            /* No warning */
+            if (!libbalsa_find_word("Recipients",
+                                    balsa_app.compose_headers)) {
+                gchar *compose_headers =
+                    g_strconcat(balsa_app.compose_headers, " Recipients",
+                                NULL);
+                g_free(balsa_app.compose_headers);
+                balsa_app.compose_headers = compose_headers;
+            }
+            libbalsa_conf_set_bool("LibBalsaAddressView", TRUE);
+        }
         libbalsa_conf_pop_group();
     }
 
@@ -1292,6 +1313,8 @@ config_save(void)
     libbalsa_conf_push_group("SourcePreview");
     libbalsa_conf_set_bool("EscapeSpecials",
                           balsa_app.source_escape_specials);
+    libbalsa_conf_set_int("Width",  balsa_app.source_width);
+    libbalsa_conf_set_int("Height", balsa_app.source_height);
     libbalsa_conf_pop_group();
 
     /* Printing options ... */
@@ -1351,9 +1374,10 @@ config_save(void)
     libbalsa_conf_push_group("Spelling");
 
 #if HAVE_GTKSPELL
-    if (balsa_app.spell_check_lang)
-        libbalsa_conf_set_string("SpellCheckLanguage",
-                                 balsa_app.spell_check_lang);
+    libbalsa_conf_set_string("SpellCheckLanguage",
+                             balsa_app.spell_check_lang);
+    libbalsa_conf_set_bool("SpellCheckActive", 
+                           balsa_app.spell_check_active);
 #else                           /* HAVE_GTKSPELL */
     libbalsa_conf_set_int("PspellModule", balsa_app.module);
     libbalsa_conf_set_int("PspellSuggestMode", balsa_app.suggestion_mode);
@@ -1375,10 +1399,14 @@ config_save(void)
     /* Maibox checking options ... */
     libbalsa_conf_push_group("MailboxChecking");
 
-	libbalsa_conf_set_int("NewMailNotificationDialog",
-				balsa_app.notify_new_mail_dialog);
-	libbalsa_conf_set_int("NewMailNotificationSound",
-				balsa_app.notify_new_mail_sound);
+    libbalsa_conf_set_int("NewMailNotificationDialog",
+                          balsa_app.notify_new_mail_dialog);
+    libbalsa_conf_set_int("NewMailNotificationSound",
+                          balsa_app.notify_new_mail_sound);
+#if GTK_CHECK_VERSION(2, 10, 0)
+    libbalsa_conf_set_int("NewMailNotificationIcon",
+                          balsa_app.notify_new_mail_icon);
+#endif                          /* GTK_CHECK_VERSION(2, 10, 0) */
     libbalsa_conf_set_bool("OnStartup", balsa_app.check_mail_upon_startup);
     libbalsa_conf_set_bool("Auto", balsa_app.check_mail_auto);
     libbalsa_conf_set_int("AutoDelay", balsa_app.check_mail_timer);
@@ -1809,22 +1837,33 @@ config_views_load(void)
                                 GINT_TO_POINTER(FALSE));
 }
 
+/* Get viewByUrl prefix */
+static gchar *
+view_by_url_prefix(const gchar * url)
+{
+    gchar *url_enc;
+    gchar *prefix;
+
+    url_enc = libbalsa_urlencode(url);
+    prefix = g_strconcat(VIEW_BY_URL_SECTION_PREFIX, url_enc, NULL);
+    g_free(url_enc);
+
+    return prefix;
+}
+
 /* config_views_save:
    iterates over all mailbox views.
 */
 static void
 save_view(const gchar * url, LibBalsaMailboxView * view)
 {
-    gchar *url_enc;
     gchar *prefix;
 
     if (!view || (view->in_sync && view->used))
 	return;
     view->in_sync = TRUE;
 
-    url_enc = libbalsa_urlencode(url);
-    prefix = g_strconcat(VIEW_BY_URL_SECTION_PREFIX, url_enc, NULL);
-    g_free(url_enc);
+    prefix = view_by_url_prefix(url);
 
     /* Remove the view--it will be recreated if any member needs to be
      * saved. */
@@ -1890,6 +1929,15 @@ config_views_save(void)
 			 NULL);
 }
 
+void
+config_view_remove(const gchar * url)
+{
+    gchar *prefix = view_by_url_prefix(url);
+    libbalsa_conf_remove_group(prefix);
+    g_free(prefix);
+    g_hash_table_remove(libbalsa_mailbox_view_table, url);
+}
+
 static void
 save_color(gchar * key, GdkColor * color)
 {
