@@ -26,6 +26,8 @@
 extern "C" {
 #endif				/* __cplusplus */
 
+#include "address-view.h"
+
     typedef enum {
        SEND_NORMAL,            /* initialized by Compose */
        SEND_REPLY,             /* by Reply               */
@@ -48,39 +50,23 @@ extern "C" {
 #define VIEW_MENU_LENGTH 5
 #endif
     typedef struct _BalsaSendmsg BalsaSendmsg;
-    typedef struct _BalsaSendmsgAddress BalsaSendmsgAddress;
-
-    struct _BalsaSendmsgAddress {
-        BalsaSendmsg *bsmsg;
-        GtkWidget *label;
-        gint min_addresses, max_addresses;
-        gboolean ready;
-    };
 
     struct _BalsaSendmsg {
 	GtkWidget *window;
-	GtkWidget *to[3], *from[3], *subject[2], *cc[3], *bcc[3], *fcc[3];
-        BalsaSendmsgAddress to_info, cc_info, bcc_info;
-#if !defined(ENABLE_TOUCH_UI)        
-        GtkWidget *reply_to[3];
-        BalsaSendmsgAddress reply_to_info;
-#endif
+        LibBalsaAddressView *recipient_view, *replyto_view;
+	GtkWidget *from[2], *recipients[2], *replyto[2], *subject[2], *fcc[2];
 	GtkWidget *attachments[4];
         gchar *in_reply_to;
         GList *references;
 	GtkWidget *text;
 #if !HAVE_GTKSPELL
 	GtkWidget *spell_checker;
-#else
-        gboolean   spell_check_error;
 #endif                          /* HAVE_GTKSPELL */
 	GtkWidget *notebook;
 	LibBalsaMessage *parent_message; /* to which we're replying     */
 	LibBalsaMessage *draft_message;  /* where the message was saved */
 	SendType type;
 	/* language selection related data */
-	gchar *charset;
-	GSList *charsets;
 	const gchar *spell_check_lang;
 	GtkWidget *current_language_menu;
 	/* identity related data */
@@ -101,6 +87,7 @@ extern "C" {
         SendmsgState state;
         gulong identities_changed_id;
 	gboolean flow;          /* send format=flowed */ 
+	gboolean send_mp_alt;   /* send multipart/alternative (plain and html) */ 
 	gboolean req_dispnotify; /* send a MDN */ 
 	gboolean quit_on_close; /* quit balsa after the compose window */
 	                        /* is closed.                          */
@@ -127,6 +114,8 @@ extern "C" {
     };
 
     BalsaSendmsg *sendmsg_window_compose(void);
+    BalsaSendmsg *sendmsg_window_compose_with_address(const gchar *
+                                                      address);
     BalsaSendmsg *sendmsg_window_reply(LibBalsaMailbox *,
                                        guint msgno, SendType rt);
     BalsaSendmsg *sendmsg_window_reply_embedded(LibBalsaMessageBody *part,
@@ -140,7 +129,8 @@ extern "C" {
     void sendmsg_window_set_field(BalsaSendmsg *bsmsg, const gchar* key,
                                   const gchar* val);
 
-    gboolean add_attachment(BalsaSendmsg * bsmsg, char *filename, 
+    gboolean add_attachment(BalsaSendmsg * bsmsg,
+                            const gchar *filename, 
                             gboolean is_a_tmp_file, 
                             const gchar *forced_mime_type);
 
