@@ -32,13 +32,14 @@
 #include <string.h>
 #include <glib/gi18n.h>
 
-#ifdef HAVE_PCRE
-#  include <pcre.h>
-#  include <pcreposix.h>
-#else
-#  include <sys/types.h>
-#  include <regex.h>
-#endif
+#if !USE_GREGEX
+#  ifdef HAVE_PCRE
+#    include <pcreposix.h>
+#  else
+#    include <sys/types.h>
+#    include <regex.h>
+#  endif
+#endif                          /* USE_GREGEX */
 
 #include "balsa-app.h"
 #include "quote-color.h"
@@ -803,7 +804,11 @@ done_cb(GtkButton * button, gpointer data)
  * PSpellManager to do the checking.
  * */
 
+#if USE_GREGEX
+static GRegex *quoted_rex;
+#else                           /* USE_GREGEX */
 static regex_t quoted_rex;
+#endif                          /* USE_GREGEX */
 static gboolean quoted_rex_compiled = FALSE;
 
 void
@@ -815,7 +820,6 @@ balsa_spell_check_start(BalsaSpellCheck * spell_check, GtkWindow *parent_wnd)
     GtkTextIter start, end, iter;
     GtkTextMark *insert;
 
-
     /* Config the spell check */
     spell_check->spell_config = new_pspell_config();
     pspell_config_replace(spell_check->spell_config,
@@ -887,6 +891,15 @@ balsa_spell_check_start(BalsaSpellCheck * spell_check, GtkWindow *parent_wnd)
      * balsa_app.quote_regex may change, so compile it new every
      * time!)
      */
+#if USE_GREGEX
+    if (quoted_rex_compiled)
+        g_regex_unref(quoted_rex);
+    quoted_rex = balsa_quote_regex_new();
+    if (!quoted_rex)
+        quoted_rex_compiled = FALSE;
+    else
+        quoted_rex_compiled = TRUE;
+#else                           /* USE_GREGEX */
     if (quoted_rex_compiled)
         regfree(&quoted_rex);
     if (regcomp(&quoted_rex, balsa_app.quote_regex, REG_EXTENDED)) {
@@ -896,6 +909,7 @@ balsa_spell_check_start(BalsaSpellCheck * spell_check, GtkWindow *parent_wnd)
         quoted_rex_compiled = FALSE;
     } else
         quoted_rex_compiled = TRUE;
+#endif                          /* USE_GREGEX */
 
     spell_check->end_iter = start;
 
@@ -1129,7 +1143,11 @@ balsa_spell_check_destroy(GtkObject * object)
     }
 
     if (quoted_rex_compiled) {
+#if USE_GREGEX
+        g_regex_unref(quoted_rex);
+#else                           /* USE_GREGEX */
         regfree(&quoted_rex);
+#endif                          /* USE_GREGEX */
         quoted_rex_compiled = FALSE;
     }
 
@@ -1366,8 +1384,13 @@ next_word(BalsaSpellCheck * spell_check)
                                         &line_end, FALSE);
             skip_sig = (!balsa_app.check_sig
                         && strcmp(line, "-- \n") == 0);
+#if USE_GREGEX
+            skip_quoted = (!balsa_app.check_quoted && quoted_rex_compiled
+                           && is_a_quote(line, quoted_rex));
+#else                           /* USE_GREGEX */
             skip_quoted = (!balsa_app.check_quoted && quoted_rex_compiled
                            && is_a_quote(line, &quoted_rex));
+#endif                          /* USE_GREGEX */
             g_free(line);
 
             if (skip_sig)
